@@ -2,6 +2,7 @@
  * Copyright (c) 1989 - 1994, Julianne Frances Haugh
  * Copyright (c) 1996 - 1998, Marek Michałkiewicz
  * Copyright (c) 2002 - 2005, Tomasz Kłoczko
+ * Copyright (c) 2008       , Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,10 +32,11 @@
 
 #include <config.h>
 
-#ident "$Id: failure.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+#ident "$Id: failure.c 2187 2008-06-15 19:16:34Z nekral-guest $"
 
 #include <fcntl.h>
 #include <stdio.h>
+#include <unistd.h>
 #include "defines.h"
 #include "faillog.h"
 #include "getdef.h"
@@ -49,13 +51,23 @@
 void failure (uid_t uid, const char *tty, struct faillog *fl)
 {
 	int fd;
+	off_t offset_uid = (off_t) (sizeof *fl) * uid;
 
 	/*
 	 * Don't do anything if failure logging isn't set up.
 	 */
 
-	if ((fd = open (FAILLOG_FILE, O_RDWR)) < 0)
+	if (access (FAILLOG_FILE, F_OK) != 0) {
 		return;
+	}
+
+	fd = open (FAILLOG_FILE, O_RDWR);
+	if (fd < 0) {
+		SYSLOG ((LOG_WARN,
+		         "Can't write faillog entry for UID %lu in %s.",
+		         (unsigned long) uid, FAILLOG_FILE));
+		return;
+	}
 
 	/*
 	 * The file is indexed by UID value meaning that shared UID's
@@ -63,9 +75,17 @@ void failure (uid_t uid, const char *tty, struct faillog *fl)
 	 * share just about everything else ...
 	 */
 
-	lseek (fd, (off_t) (sizeof *fl) * uid, SEEK_SET);
-	if (read (fd, (char *) fl, sizeof *fl) != sizeof *fl)
+	if (   (lseek (fd, offset_uid, SEEK_SET) != offset_uid)
+	    || (read (fd, (char *) fl, sizeof *fl) != (ssize_t) sizeof *fl)) {
+		/* This is not necessarily a failure. The file is
+		 * initially zero length.
+		 *
+		 * If lseek() or read() failed for any other reason, this
+		 * might reset the counter. But the new failure will be
+		 * logged.
+		 */
 		memzero (fl, sizeof *fl);
+	}
 
 	/*
 	 * Update the record.  We increment the failure count to log the
@@ -74,11 +94,12 @@ void failure (uid_t uid, const char *tty, struct faillog *fl)
 	 * updated as well.
 	 */
 
-	if (fl->fail_cnt + 1 > 0)
+	if (fl->fail_cnt + 1 > 0) {
 		fl->fail_cnt++;
+	}
 
 	strncpy (fl->fail_line, tty, sizeof fl->fail_line);
-	time (&fl->fail_time);
+	(void) time (&fl->fail_time);
 
 	/*
 	 * Seek back to the correct position in the file and write the
@@ -87,26 +108,34 @@ void failure (uid_t uid, const char *tty, struct faillog *fl)
 	 * seem that great.
 	 */
 
-	lseek (fd, (off_t) (sizeof *fl) * uid, SEEK_SET);
-	write (fd, (char *) fl, sizeof *fl);
-	close (fd);
+	if (   (lseek (fd, offset_uid, SEEK_SET) != offset_uid)
+	    || (write (fd, (char *) fl, sizeof *fl) != (ssize_t) sizeof *fl)
+	    || (close (fd) != 0)) {
+		SYSLOG ((LOG_WARN,
+		         "Can't write faillog entry for UID %lu in %s.",
+		         (unsigned long) uid, FAILLOG_FILE));
+		(void) close (fd);
+	}
 }
 
-static int too_many_failures (const struct faillog *fl)
+static bool too_many_failures (const struct faillog *fl)
 {
 	time_t now;
 
-	if (fl->fail_max == 0 || fl->fail_cnt < fl->fail_max)
-		return 0;
+	if ((0 == fl->fail_max) || (fl->fail_cnt < fl->fail_max)) {
+		return false;
+	}
 
-	if (fl->fail_locktime == 0)
-		return 1;	/* locked until reset manually */
+	if (0 == fl->fail_locktime) {
+		return true;	/* locked until reset manually */
+	}
 
-	time (&now);
-	if (fl->fail_time + fl->fail_locktime < now)
-		return 0;	/* enough time since last failure */
+	(void) time (&now);
+	if ((fl->fail_time + fl->fail_locktime) < now) {
+		return false;	/* enough time since last failure */
+	}
 
-	return 1;
+	return true;
 }
 
 /*
@@ -114,21 +143,35 @@ static int too_many_failures (const struct faillog *fl)
  *
  *	failcheck() is called AFTER the password has been validated.  If the
  *	account has been "attacked" with too many login failures, failcheck()
- *	returns FALSE to indicate that the login should be denied even though
+ *	returns 0 to indicate that the login should be denied even though
  *	the password is valid.
+ *
+ *	failed indicates if the login failed AFTER the password has been
+ *	       validated.
  */
 
-int failcheck (uid_t uid, struct faillog *fl, int failed)
+int failcheck (uid_t uid, struct faillog *fl, bool failed)
 {
 	int fd;
 	struct faillog fail;
+	off_t offset_uid = (off_t) (sizeof *fl) * uid;
 
 	/*
 	 * Suppress the check if the log file isn't there.
 	 */
 
-	if ((fd = open (FAILLOG_FILE, O_RDWR)) < 0)
+	if (access (FAILLOG_FILE, F_OK) != 0) {
 		return 1;
+	}
+
+	fd = open (FAILLOG_FILE, failed?O_RDONLY:O_RDWR);
+	if (fd < 0) {
+		SYSLOG ((LOG_WARN,
+		         "Can't open the faillog file (%s) to check UID %lu. "
+		         "User access authorized.",
+		         FAILLOG_FILE, (unsigned long) uid));
+		return 1;
+	}
 
 	/*
 	 * Get the record from the file and determine if the user has
@@ -142,14 +185,14 @@ int failcheck (uid_t uid, struct faillog *fl, int failed)
 	 * no need to reset the count.
 	 */
 
-	lseek (fd, (off_t) (sizeof *fl) * uid, SEEK_SET);
-	if (read (fd, (char *) fl, sizeof *fl) != sizeof *fl) {
-		close (fd);
+	if (   (lseek (fd, offset_uid, SEEK_SET) != offset_uid)
+	    || (read (fd, (char *) fl, sizeof *fl) != (ssize_t) sizeof *fl)) {
+		(void) close (fd);
 		return 1;
 	}
 
 	if (too_many_failures (fl)) {
-		close (fd);
+		(void) close (fd);
 		return 0;
 	}
 
@@ -164,10 +207,18 @@ int failcheck (uid_t uid, struct faillog *fl, int failed)
 		fail = *fl;
 		fail.fail_cnt = 0;
 
-		lseek (fd, (off_t) sizeof fail * uid, SEEK_SET);
-		write (fd, (char *) &fail, sizeof fail);
+		if (   (lseek (fd, offset_uid, SEEK_SET) != offset_uid)
+		    || (write (fd, (const void *) &fail, sizeof fail) != (ssize_t) sizeof fail)
+		    || (close (fd) != 0)) {
+			SYSLOG ((LOG_WARN,
+			         "Can't reset faillog entry for UID %lu in %s.",
+			         (unsigned long) uid, FAILLOG_FILE));
+			(void) close (fd);
+		}
+	} else {
+		(void) close (fd);
 	}
-	close (fd);
+
 	return 1;
 }
 
@@ -190,17 +241,18 @@ void failprint (const struct faillog *fail)
 #endif
 	time_t NOW;
 
-	if (fail->fail_cnt == 0)
+	if (0 == fail->fail_cnt) {
 		return;
+	}
 
 	tp = localtime (&(fail->fail_time));
-	time (&NOW);
+	(void) time (&NOW);
 
 #if HAVE_STRFTIME
 	/*
 	 * Print all information we have.
 	 */
-	strftime (lasttimeb, sizeof lasttimeb, "%c", tp);
+	(void) strftime (lasttimeb, sizeof lasttimeb, "%c", tp);
 #else
 
 	/*
@@ -210,20 +262,23 @@ void failprint (const struct faillog *fail)
 	lasttime = asctime (tp);
 	lasttime[24] = '\0';
 
-	if (NOW - fail->fail_time < YEAR)
+	if ((NOW - fail->fail_time) < YEAR) {
 		lasttime[19] = '\0';
-	if (NOW - fail->fail_time < DAY)
+	}
+	if ((NOW - fail->fail_time) < DAY) {
 		lasttime = lasttime + 11;
+	}
 
-	if (*lasttime == ' ')
+	if (' ' == *lasttime) {
 		lasttime++;
+	}
 #endif
-	printf (ngettext ("%d failure since last login.\n"
-			  "Last was %s on %s.\n",
-			  "%d failures since last login.\n"
-			  "Last was %s on %s.\n",
-			  fail->fail_cnt),
-		fail->fail_cnt, lasttime, fail->fail_line);
+	(void) printf (ngettext ("%d failure since last login.\n"
+	                         "Last was %s on %s.\n",
+	                         "%d failures since last login.\n"
+	                         "Last was %s on %s.\n",
+	                         (unsigned long) fail->fail_cnt),
+	               fail->fail_cnt, lasttime, fail->fail_line);
 }
 
 /*
@@ -249,21 +304,46 @@ void failtmp (
 	 * in login.defs, don't do this.
 	 */
 
-	if (!(ftmp = getdef_str ("FTMP_FILE")))
+	ftmp = getdef_str ("FTMP_FILE");
+	if (NULL == ftmp) {
 		return;
+	}
 
 	/*
 	 * Open the file for append.  It must already exist for this
 	 * feature to be used.
 	 */
 
-	if ((fd = open (ftmp, O_WRONLY | O_APPEND)) == -1)
+	if (access (ftmp, F_OK) != 0) {
 		return;
+	}
+
+	fd = open (ftmp, O_WRONLY | O_APPEND);
+	if (-1 == fd) {
+		char ut_user[sizeof failent->ut_user];
+		(void) strncpy (&ut_user[0], failent->ut_user, sizeof ut_user);
+		ut_user[sizeof ut_user - 1] = '\0';
+
+		SYSLOG ((LOG_WARN,
+		         "Can't append failure of user %s to %s.",
+		         ut_user, ftmp));
+		return;
+	}
 
 	/*
-	 * Output the new failure record and close the log file.
+	 * Append the new failure record and close the log file.
 	 */
 
-	write (fd, (const char *) failent, sizeof *failent);
-	close (fd);
+	if (   (write (fd, (const void *) failent, sizeof *failent) != (ssize_t) sizeof *failent)
+	    || (close (fd) != 0)) {
+		char ut_user[sizeof failent->ut_user];
+		(void) strncpy (&ut_user[0], failent->ut_user, sizeof ut_user);
+		ut_user[sizeof ut_user - 1] = '\0';
+
+		SYSLOG ((LOG_WARN,
+		         "Can't append failure of user %s to %s.",
+		         ut_user, ftmp));
+		(void) close (fd);
+	}
 }
+
