@@ -2,7 +2,7 @@
  * Copyright (c) 1989 - 1994, Julianne Frances Haugh
  * Copyright (c) 1996 - 2000, Marek Michałkiewicz
  * Copyright (c) 2001 - 2006, Tomasz Kłoczko
- * Copyright (c) 2007       , Nicolas François
+ * Copyright (c) 2007 - 2008, Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@
 
 #include <config.h>
 
-#ident "$Id: setupenv.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+#ident "$Id: setupenv.c 2046 2008-05-26 00:04:30Z nekral-guest $"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -229,7 +229,7 @@ void setup_env (struct passwd *info)
 	 * Create the SHELL environmental variable and export it.
 	 */
 
-	if (info->pw_shell == (char *) 0 || !*info->pw_shell) {
+	if ((NULL == info->pw_shell) || ('\0' == *info->pw_shell)) {
 		static char temp_pw_shell[] = "/bin/sh";
 
 		info->pw_shell = temp_pw_shell;
