@@ -32,7 +32,7 @@
 
 #include <config.h>
 
-#ident "$Id: chfn.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+#ident "$Id: chfn.c 2363 2008-09-06 23:46:44Z nekral-guest $"
 
 #include <fcntl.h>
 #include <pwd.h>
@@ -56,30 +56,29 @@
 /*
  * Global variables.
  */
-static char *Prog;
+char *Prog;
 static char fullnm[BUFSIZ];
 static char roomno[BUFSIZ];
 static char workph[BUFSIZ];
 static char homeph[BUFSIZ];
 static char slop[BUFSIZ];
-static int amroot;
+static bool amroot;
 /* Flags */
-static int fflg = 0;		/* -f - set full name                */
-static int rflg = 0;		/* -r - set room number              */
-static int wflg = 0;		/* -w - set work phone number        */
-static int hflg = 0;		/* -h - set home phone number        */
-static int oflg = 0;		/* -o - set other information        */
-#ifdef USE_PAM
-static pam_handle_t *pamh = NULL;
-#endif
+static bool fflg = false;		/* -f - set full name                */
+static bool rflg = false;		/* -r - set room number              */
+static bool wflg = false;		/* -w - set work phone number        */
+static bool hflg = false;		/* -h - set home phone number        */
+static bool oflg = false;		/* -o - set other information        */
+static bool pw_locked = false;
 
 /*
  * External identifiers
  */
 
 /* local function prototypes */
+static void fail_exit (int code);
 static void usage (void);
-static int may_change_field (int);
+static bool may_change_field (int);
 static void new_fields (void);
 static char *copy_field (char *, char *, char *);
 static void process_flags (int argc, char **argv);
@@ -87,6 +86,25 @@ static void check_perms (const struct passwd *pw);
 static void update_gecos (const char *user, char *gecos);
 static void get_old_fields (const char *gecos);
 
+/*
+ * fail_exit - exit with an error and do some cleanup
+ */
+static void fail_exit (int code)
+{
+	if (pw_locked) {
+		if (pw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+			/* continue */
+		}
+	}
+	pw_locked = false;
+
+	closelog ();
+
+	exit (code);
+}
+
 /*
  * usage - print command line syntax and exit
  */
@@ -113,9 +131,9 @@ static void usage (void)
  *
  *	field should be one of 'f', 'r', 'w', 'h'
  *
- *	Return 1 if the user can change the field and 0 otherwise.
+ *	Return true if the user can change the field and false otherwise.
  */
-static int may_change_field (int field)
+static bool may_change_field (int field)
 {
 	const char *cp;
 
@@ -137,7 +155,7 @@ static int may_change_field (int field)
 	 * if the string is empty or not defined at all.
 	 */
 	if (amroot) {
-		return 1;
+		return true;
 	}
 
 	cp = getdef_str ("CHFN_RESTRICT");
@@ -150,10 +168,10 @@ static int may_change_field (int field)
 	}
 
 	if (strchr (cp, field) != NULL) {
-		return 1;
+		return true;
 	}
 
-	return 0;
+	return false;
 }
 
 /*
@@ -169,25 +187,25 @@ static void new_fields (void)
 	if (may_change_field ('f')) {
 		change_field (fullnm, sizeof fullnm, _("Full Name"));
 	} else {
-		printf (_("\tFull Name: %s\n"), fullnm);
+		printf (_("\t%s: %s\n"), _("Full Name"), fullnm);
 	}
 
 	if (may_change_field ('r')) {
 		change_field (roomno, sizeof roomno, _("Room Number"));
 	} else {
-		printf (_("\tRoom Number: %s\n"), roomno);
+		printf (_("\t%s: %s\n"), _("Room Number"), fullnm);
 	}
 
 	if (may_change_field ('w')) {
 		change_field (workph, sizeof workph, _("Work Phone"));
 	} else {
-		printf (_("\tWork Phone: %s\n"), workph);
+		printf (_("\t%s: %s\n"), _("Work Phone"), fullnm);
 	}
 
 	if (may_change_field ('h')) {
 		change_field (homeph, sizeof homeph, _("Home Phone"));
 	} else {
-		printf (_("\tHome Phone: %s\n"), homeph);
+		printf (_("\t%s: %s\n"), _("Home Phone"), fullnm);
 	}
 
 	if (amroot) {
@@ -259,7 +277,7 @@ static void process_flags (int argc, char **argv)
 				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
-			fflg++;
+			fflg = true;
 			STRFCPY (fullnm, optarg);
 			break;
 		case 'h':
@@ -268,7 +286,7 @@ static void process_flags (int argc, char **argv)
 				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
-			hflg++;
+			hflg = true;
 			STRFCPY (homeph, optarg);
 			break;
 		case 'r':
@@ -277,7 +295,7 @@ static void process_flags (int argc, char **argv)
 				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
-			rflg++;
+			rflg = true;
 			STRFCPY (roomno, optarg);
 			break;
 		case 'o':
@@ -286,7 +304,7 @@ static void process_flags (int argc, char **argv)
 				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
-			oflg++;
+			oflg = true;
 			STRFCPY (slop, optarg);
 			break;
 		case 'w':
@@ -295,7 +313,7 @@ static void process_flags (int argc, char **argv)
 				         _("%s: Permission denied.\n"), Prog);
 				exit (E_NOPERM);
 			}
-			wflg++;
+			wflg = true;
 			STRFCPY (workph, optarg);
 			break;
 		default:
@@ -317,6 +335,7 @@ static void process_flags (int argc, char **argv)
 static void check_perms (const struct passwd *pw)
 {
 #ifdef USE_PAM
+	pam_handle_t *pamh = NULL;
 	int retval;
 	struct passwd *pampw;
 #endif
@@ -356,32 +375,28 @@ static void check_perms (const struct passwd *pw)
 	}
 
 #else				/* !USE_PAM */
-	retval = PAM_SUCCESS;
-
 	pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
+	if (NULL == pampw) {
+		fprintf (stderr,
+		         _("%s: Cannot determine your user name.\n"),
+		         Prog);
+		exit (E_NOPERM);
 	}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("chfn", pampw->pw_name, &conv, &pamh);
-	}
+	retval = pam_start ("chfn", pampw->pw_name, &conv, &pamh);
 
-	if (retval == PAM_SUCCESS) {
+	if (PAM_SUCCESS == retval) {
 		retval = pam_authenticate (pamh, 0);
-		if (retval != PAM_SUCCESS) {
-			pam_end (pamh, retval);
-		}
 	}
 
-	if (retval == PAM_SUCCESS) {
+	if (PAM_SUCCESS == retval) {
 		retval = pam_acct_mgmt (pamh, 0);
-		if (retval != PAM_SUCCESS) {
-			pam_end (pamh, retval);
-		}
 	}
 
-	if (retval != PAM_SUCCESS) {
+	if (NULL != pamh) {
+		(void) pam_end (pamh, retval);
+	}
+	if (PAM_SUCCESS != retval) {
 		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
 		exit (E_NOPERM);
 	}
@@ -407,8 +422,7 @@ static void update_gecos (const char *user, char *gecos)
 	if (setuid (0) != 0) {
 		fputs (_("Cannot change ID to root.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
-		closelog ();
-		exit (E_NOPERM);
+		fail_exit (E_NOPERM);
 	}
 	pwd_init ();
 
@@ -417,18 +431,16 @@ static void update_gecos (const char *user, char *gecos)
 	 * password file. Get a lock on the file and open it.
 	 */
 	if (pw_lock () == 0) {
-		fputs (_("Cannot lock the password file; try again later.\n"),
-		       stderr);
-		SYSLOG ((LOG_WARN, "can't lock /etc/passwd"));
-		closelog ();
-		exit (E_NOPERM);
+		fprintf (stderr,
+		         _("%s: cannot lock %s; try again later.\n"),
+		         Prog, pw_dbname ());
+		fail_exit (E_NOPERM);
 	}
+	pw_locked = true;
 	if (pw_open (O_RDWR) == 0) {
-		fputs (_("Cannot open the password file.\n"), stderr);
-		pw_unlock ();
-		SYSLOG ((LOG_ERR, "can't open /etc/passwd"));
-		closelog ();
-		exit (E_NOPERM);
+		fprintf (stderr,
+		         _("%s: cannot open %s\n"), Prog, pw_dbname ());
+		fail_exit (E_NOPERM);
 	}
 
 	/*
@@ -439,10 +451,10 @@ static void update_gecos (const char *user, char *gecos)
 	 */
 	pw = pw_locate (user);
 	if (NULL == pw) {
-		pw_unlock ();
 		fprintf (stderr,
-		         _("%s: %s not found in /etc/passwd\n"), Prog, user);
-		exit (E_NOPERM);
+		         _("%s: user '%s' does not exist in %s\n"),
+		         Prog, user, pw_dbname ());
+		fail_exit (E_NOPERM);
 	}
 
 	/*
@@ -457,29 +469,26 @@ static void update_gecos (const char *user, char *gecos)
 	 * entry as well.
 	 */
 	if (pw_update (&pwent) == 0) {
-		fputs (_("Error updating the password entry.\n"), stderr);
-		pw_unlock ();
-		SYSLOG ((LOG_ERR, "error updating passwd entry"));
-		closelog ();
-		exit (E_NOPERM);
+		fprintf (stderr,
+		         _("%s: failed to prepare the new %s entry '%s'\n"),
+		         Prog, pw_dbname (), pwent.pw_name);
+		fail_exit (E_NOPERM);
 	}
 
 	/*
 	 * Changes have all been made, so commit them and unlock the file.
 	 */
 	if (pw_close () == 0) {
-		fputs (_("Cannot commit password file changes.\n"), stderr);
-		pw_unlock ();
-		SYSLOG ((LOG_ERR, "can't rewrite /etc/passwd"));
-		closelog ();
-		exit (E_NOPERM);
+		fprintf (stderr, _("%s: failure while writing changes to %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", pw_dbname ()));
+		fail_exit (E_NOPERM);
 	}
 	if (pw_unlock () == 0) {
-		fputs (_("Cannot unlock the password file.\n"), stderr);
-		SYSLOG ((LOG_ERR, "can't unlock /etc/passwd"));
-		closelog ();
-		exit (E_NOPERM);
+		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+		/* continue */
 	}
+	pw_locked = false;
 }
 
 /*
@@ -545,8 +554,7 @@ static void check_fields (void)
 		fprintf (stderr, _("%s: name with non-ASCII characters: '%s'\n"), Prog, fullnm);
 	} else if (err < 0) {
 		fprintf (stderr, _("%s: invalid name: '%s'\n"), Prog, fullnm);
-		closelog ();
-		exit (E_NOPERM);
+		fail_exit (E_NOPERM);
 	}
 	err = valid_field (roomno, ":,=");
 	if (err > 0) {
@@ -554,20 +562,17 @@ static void check_fields (void)
 	} else if (err < 0) {
 		fprintf (stderr, _("%s: invalid room number: '%s'\n"),
 		         Prog, roomno);
-		closelog ();
-		exit (E_NOPERM);
+		fail_exit (E_NOPERM);
 	}
-	if (valid_field (workph, ":,=")) {
+	if (valid_field (workph, ":,=") != 0) {
 		fprintf (stderr, _("%s: invalid work phone: '%s'\n"),
 		         Prog, workph);
-		closelog ();
-		exit (E_NOPERM);
+		fail_exit (E_NOPERM);
 	}
-	if (valid_field (homeph, ":,=")) {
+	if (valid_field (homeph, ":,=") != 0) {
 		fprintf (stderr, _("%s: invalid home phone: '%s'\n"),
 		         Prog, homeph);
-		closelog ();
-		exit (E_NOPERM);
+		fail_exit (E_NOPERM);
 	}
 	err = valid_field (slop, ":");
 	if (err > 0) {
@@ -576,8 +581,7 @@ static void check_fields (void)
 		fprintf (stderr,
 		         _("%s: '%s' contains illegal characters\n"),
 		         Prog, slop);
-		closelog ();
-		exit (E_NOPERM);
+		fail_exit (E_NOPERM);
 	}
 }
 
@@ -604,9 +608,9 @@ int main (int argc, char **argv)
 	char *user;
 
 	sanitize_env ();
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
+	(void) setlocale (LC_ALL, "");
+	(void) bindtextdomain (PACKAGE, LOCALEDIR);
+	(void) textdomain (PACKAGE);
 
 	/*
 	 * This command behaves different for root and non-root
@@ -633,18 +637,19 @@ int main (int argc, char **argv)
 		user = argv[optind];
 		pw = xgetpwnam (user);
 		if (NULL == pw) {
-			fprintf (stderr, _("%s: unknown user %s\n"), Prog,
+			fprintf (stderr, _("%s: user '%s' does not exist\n"), Prog,
 			         user);
-			exit (E_NOPERM);
+			fail_exit (E_NOPERM);
 		}
 	} else {
 		pw = get_my_pwent ();
 		if (NULL == pw) {
 			fprintf (stderr,
-			         _
-			         ("%s: Cannot determine your user name.\n"),
+			         _("%s: Cannot determine your user name.\n"),
 			         Prog);
-			exit (E_NOPERM);
+			SYSLOG ((LOG_WARN, "Cannot determine the user name of the caller (UID %lu)",
+			         (unsigned long) getuid ()));
+			fail_exit (E_NOPERM);
 		}
 		user = xstrdup (pw->pw_name);
 	}
@@ -668,7 +673,7 @@ int main (int argc, char **argv)
 			         ("%s: '%s' is the NIS master for this client.\n"),
 			         Prog, nis_master);
 		}
-		exit (E_NOPERM);
+		fail_exit (E_NOPERM);
 	}
 #endif
 
@@ -701,23 +706,19 @@ int main (int argc, char **argv)
 	if ((strlen (fullnm) + strlen (roomno) + strlen (workph) +
 	     strlen (homeph) + strlen (slop)) > (unsigned int) 80) {
 		fprintf (stderr, _("%s: fields too long\n"), Prog);
-		closelog ();
-		exit (E_NOPERM);
+		fail_exit (E_NOPERM);
 	}
 	snprintf (new_gecos, sizeof new_gecos, "%s,%s,%s,%s%s%s",
-		  fullnm, roomno, workph, homeph, slop[0] ? "," : "", slop);
+	          fullnm, roomno, workph, homeph,
+	          ('\0' != slop[0]) ? "," : "", slop);
 
 	/* Rewrite the user's gecos in the passwd file */
 	update_gecos (user, new_gecos);
 
-	SYSLOG ((LOG_INFO, "changed user `%s' information", user));
+	SYSLOG ((LOG_INFO, "changed user '%s' information", user));
 
 	nscd_flush_cache ("passwd");
 
-#ifdef USE_PAM
-	pam_end (pamh, PAM_SUCCESS);
-#endif				/* USE_PAM */
-
 	closelog ();
 	exit (E_SUCCESS);
 }
