@@ -32,12 +32,11 @@
 
 #include <config.h>
 
-#ident "$Id: chsh.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+#ident "$Id: chsh.c 2363 2008-09-06 23:46:44Z nekral-guest $"
 
 #include <fcntl.h>
 #include <getopt.h>
 #include <pwd.h>
-#include <signal.h>
 #include <stdio.h>
 #include <sys/types.h>
 #ifdef WITH_SELINUX
@@ -60,26 +59,43 @@
 /*
  * Global variables
  */
-static char *Prog;		/* Program name */
-static int amroot;		/* Real UID is root */
+char *Prog;		/* Program name */
+static bool amroot;		/* Real UID is root */
 static char loginsh[BUFSIZ];	/* Name of new login shell */
 /* command line options */
-static int sflg = 0;		/* -s - set shell from command line  */
-#ifdef USE_PAM
-static pam_handle_t *pamh = NULL;
-#endif
+static bool sflg = false;	/* -s - set shell from command line  */
+static bool pw_locked = false;
 
 /* external identifiers */
 
 /* local function prototypes */
+static void fail_exit (int code);
 static void usage (void);
 static void new_fields (void);
-static int check_shell (const char *);
-static int restricted_shell (const char *);
+static bool shell_is_listed (const char *);
+static bool is_restricted_shell (const char *);
 static void process_flags (int argc, char **argv);
 static void check_perms (const struct passwd *pw);
 static void update_shell (const char *user, char *loginsh);
 
+/*
+ * fail_exit - do some cleanup and exit with the given error code
+ */
+static void fail_exit (int code)
+{
+	if (pw_locked) {
+		if (pw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+			/* continue */
+		}
+	}
+
+	closelog ();
+
+	exit (code);
+}
+
 /*
  * usage - print command line syntax and exit
  */
@@ -107,23 +123,21 @@ static void new_fields (void)
 }
 
 /*
- * restricted_shell - return true if the named shell begins with 'r' or 'R'
+ * is_restricted_shell - return true if the shell is restricted
  *
- * If the first letter of the filename is 'r' or 'R', the shell is
- * considered to be restricted.
  */
-static int restricted_shell (const char *sh)
+static bool is_restricted_shell (const char *sh)
 {
 	/*
 	 * Shells not listed in /etc/shells are considered to be restricted.
 	 * Changed this to avoid confusion with "rc" (the plan9 shell - not
 	 * restricted despite the name starting with 'r').  --marekm
 	 */
-	return !check_shell (sh);
+	return !shell_is_listed (sh);
 }
 
 /*
- * check_shell - see if the user's login shell is listed in /etc/shells
+ * shell_is_listed - see if the user's login shell is listed in /etc/shells
  *
  * The /etc/shells file is read for valid names of login shells.  If the
  * /etc/shells file does not exist the user cannot set any shell unless
@@ -132,10 +146,10 @@ static int restricted_shell (const char *sh)
  * If getusershell() is available (Linux, *BSD, possibly others), use it
  * instead of re-implementing it.
  */
-static int check_shell (const char *sh)
+static bool shell_is_listed (const char *sh)
 {
 	char *cp;
-	int found = 0;
+	bool found = false;
 
 #ifndef HAVE_GETUSERSHELL
 	char buf[BUFSIZ];
@@ -150,7 +164,7 @@ static int check_shell (const char *sh)
 		}
 
 		if (strcmp (cp, sh) == 0) {
-			found = 1;
+			found = true;
 			break;
 		}
 	}
@@ -158,10 +172,10 @@ static int check_shell (const char *sh)
 #else
 	fp = fopen (SHELLS_FILE, "r");
 	if (NULL == fp) {
-		return 0;
+		return false;
 	}
 
-	while (fgets (buf, sizeof (buf), fp)) {
+	while (fgets (buf, sizeof (buf), fp) == buf) {
 		cp = strrchr (buf, '\n');
 		if (NULL != cp) {
 			*cp = '\0';
@@ -172,7 +186,7 @@ static int check_shell (const char *sh)
 		}
 
 		if (strcmp (buf, sh) == 0) {
-			found = 1;
+			found = true;
 			break;
 		}
 	}
@@ -204,7 +218,7 @@ static void process_flags (int argc, char **argv)
 			usage ();
 			break;
 		case 's':
-			sflg++;
+			sflg = true;
 			STRFCPY (loginsh, optarg);
 			break;
 		default:
@@ -234,6 +248,7 @@ static void process_flags (int argc, char **argv)
 static void check_perms (const struct passwd *pw)
 {
 #ifdef USE_PAM
+	pam_handle_t *pamh = NULL;
 	int retval;
 	struct passwd *pampw;
 #endif
@@ -243,25 +258,23 @@ static void check_perms (const struct passwd *pw)
 	 * UID of the user matches the current real UID.
 	 */
 	if (!amroot && pw->pw_uid != getuid ()) {
-		SYSLOG ((LOG_WARN, "can't change shell for `%s'", pw->pw_name));
-		closelog ();
+		SYSLOG ((LOG_WARN, "can't change shell for '%s'", pw->pw_name));
 		fprintf (stderr,
-		         _("You may not change the shell for %s.\n"),
+		         _("You may not change the shell for '%s'.\n"),
 		         pw->pw_name);
-		exit (1);
+		fail_exit (1);
 	}
 
 	/*
 	 * Non-privileged users are only allowed to change the shell if it
 	 * is not a restricted one.
 	 */
-	if (!amroot && restricted_shell (pw->pw_shell)) {
-		SYSLOG ((LOG_WARN, "can't change shell for `%s'", pw->pw_name));
-		closelog ();
+	if (!amroot && is_restricted_shell (pw->pw_shell)) {
+		SYSLOG ((LOG_WARN, "can't change shell for '%s'", pw->pw_name));
 		fprintf (stderr,
-		         _("You may not change the shell for %s.\n"),
+		         _("You may not change the shell for '%s'.\n"),
 		         pw->pw_name);
-		exit (1);
+		fail_exit (1);
 	}
 #ifdef WITH_SELINUX
 	/*
@@ -271,12 +284,11 @@ static void check_perms (const struct passwd *pw)
 	if ((pw->pw_uid != getuid ())
 	    && (is_selinux_enabled () > 0)
 	    && (selinux_check_passwd_access (PASSWD__CHSH) != 0)) {
-		SYSLOG ((LOG_WARN, "can't change shell for `%s'", pw->pw_name));
-		closelog ();
+		SYSLOG ((LOG_WARN, "can't change shell for '%s'", pw->pw_name));
 		fprintf (stderr,
-		         _("You may not change the shell for %s.\n"),
+		         _("You may not change the shell for '%s'.\n"),
 		         pw->pw_name);
-		exit (1);
+		fail_exit (1);
 	}
 #endif
 
@@ -292,32 +304,28 @@ static void check_perms (const struct passwd *pw)
         }
 
 #else				/* !USE_PAM */
-	retval = PAM_SUCCESS;
-
 	pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
+	if (NULL == pampw) {
+		fprintf (stderr,
+		         _("%s: Cannot determine your user name.\n"),
+		         Prog);
+		exit (E_NOPERM);
 	}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("chsh", pampw->pw_name, &conv, &pamh);
-	}
+	retval = pam_start ("chsh", pampw->pw_name, &conv, &pamh);
 
-	if (retval == PAM_SUCCESS) {
+	if (PAM_SUCCESS == retval) {
 		retval = pam_authenticate (pamh, 0);
-		if (retval != PAM_SUCCESS) {
-			pam_end (pamh, retval);
-		}
 	}
 
-	if (retval == PAM_SUCCESS) {
+	if (PAM_SUCCESS == retval) {
 		retval = pam_acct_mgmt (pamh, 0);
-		if (retval != PAM_SUCCESS) {
-			pam_end (pamh, retval);
-		}
 	}
 
-	if (retval != PAM_SUCCESS) {
+	if (NULL != pamh) {
+		(void) pam_end (pamh, retval);
+	}
+	if (PAM_SUCCESS != retval) {
 		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
 		exit (E_NOPERM);
 	}
@@ -344,9 +352,8 @@ static void update_shell (const char *user, char *newshell)
 	 */
 	if (setuid (0) != 0) {
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
-		closelog ();
 		fputs (_("Cannot change ID to root.\n"), stderr);
-		exit (1);
+		fail_exit (1);
 	}
 	pwd_init ();
 
@@ -355,18 +362,15 @@ static void update_shell (const char *user, char *newshell)
 	 * the password file. Get a lock on the file and open it.
 	 */
 	if (pw_lock () == 0) {
-		SYSLOG ((LOG_WARN, "can't lock /etc/passwd"));
-		closelog ();
-		fputs (_("Cannot lock the password file; try again later.\n"),
-		       stderr);
-		exit (1);
+		fprintf (stderr, _("%s: cannot lock %s; try again later.\n"),
+		         Prog, pw_dbname ());
+		fail_exit (1);
 	}
+	pw_locked = true;
 	if (pw_open (O_RDWR) == 0) {
-		SYSLOG ((LOG_ERR, "can't open /etc/passwd"));
-		closelog ();
-		fputs (_("Cannot open the password file.\n"), stderr);
-		pw_unlock ();
-		exit (1);
+		fprintf (stderr, _("%s: cannot open %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_WARN, "cannot open %s", pw_dbname ()));
+		fail_exit (1);
 	}
 
 	/*
@@ -377,10 +381,10 @@ static void update_shell (const char *user, char *newshell)
 	 */
 	pw = pw_locate (user);
 	if (NULL == pw) {
-		pw_unlock ();
 		fprintf (stderr,
-		         _("%s: %s not found in /etc/passwd\n"), Prog, user);
-		exit (1);
+		         _("%s: user '%s' does not exist in %s\n"),
+		         Prog, user, pw_dbname ());
+		fail_exit (1);
 	}
 
 	/*
@@ -395,29 +399,26 @@ static void update_shell (const char *user, char *newshell)
 	 * that entry as well.
 	 */
 	if (pw_update (&pwent) == 0) {
-		SYSLOG ((LOG_ERR, "error updating passwd entry"));
-		closelog ();
-		fputs (_("Error updating the password entry.\n"), stderr);
-		pw_unlock ();
-		exit (1);
+		fprintf (stderr,
+		         _("%s: failed to prepare the new %s entry '%s'\n"),
+		         Prog, pw_dbname (), pwent.pw_name);
+		fail_exit (1);
 	}
 
 	/*
 	 * Changes have all been made, so commit them and unlock the file.
 	 */
 	if (pw_close () == 0) {
-		SYSLOG ((LOG_ERR, "can't rewrite /etc/passwd"));
-		closelog ();
-		fputs (_("Cannot commit password file changes.\n"), stderr);
-		pw_unlock ();
-		exit (1);
+		fprintf (stderr, _("%s: failure while writing changes to %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", pw_dbname ()));
+		fail_exit (1);
 	}
 	if (pw_unlock () == 0) {
-		SYSLOG ((LOG_ERR, "can't unlock /etc/passwd"));
-		closelog ();
-		fputs (_("Cannot unlock the password file.\n"), stderr);
-		exit (1);
+		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+		/* continue */
 	}
+	pw_locked= false;
 }
 
 /*
@@ -433,14 +434,14 @@ int main (int argc, char **argv)
 
 	sanitize_env ();
 
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
+	(void) setlocale (LC_ALL, "");
+	(void) bindtextdomain (PACKAGE, LOCALEDIR);
+	(void) textdomain (PACKAGE);
 
 	/*
 	 * This command behaves different for root and non-root users.
 	 */
-	amroot = getuid () == 0;
+	amroot = (getuid () == 0);
 
 	/*
 	 * Get the program name. The program name is used as a prefix to
@@ -460,19 +461,20 @@ int main (int argc, char **argv)
 	if (optind < argc) {
 		user = argv[optind];
 		pw = xgetpwnam (user);
-		if (!pw) {
+		if (NULL == pw) {
 			fprintf (stderr,
-			         _("%s: unknown user %s\n"), Prog, user);
-			exit (1);
+			         _("%s: user '%s' does not exist\n"), Prog, user);
+			fail_exit (1);
 		}
 	} else {
 		pw = get_my_pwent ();
-		if (!pw) {
+		if (NULL == pw) {
 			fprintf (stderr,
-			         _
-			         ("%s: Cannot determine your user name.\n"),
+			         _("%s: Cannot determine your user name.\n"),
 			         Prog);
-			exit (1);
+			SYSLOG ((LOG_WARN, "Cannot determine the user name of the caller (UID %lu)",
+			         (unsigned long) getuid ()));
+			fail_exit (1);
 		}
 		user = xstrdup (pw->pw_name);
 	}
@@ -492,11 +494,10 @@ int main (int argc, char **argv)
 		if (!yp_get_default_domain (&nis_domain) &&
 		    !yp_master (nis_domain, "passwd.byname", &nis_master)) {
 			fprintf (stderr,
-			         _
-			         ("%s: '%s' is the NIS master for this client.\n"),
+			         _("%s: '%s' is the NIS master for this client.\n"),
 			         Prog, nis_master);
 		}
-		exit (1);
+		fail_exit (1);
 	}
 #endif
 
@@ -525,27 +526,23 @@ int main (int argc, char **argv)
 	 * users are restricted to using the shells in /etc/shells.
 	 * The shell must be executable by the user.
 	 */
-	if (valid_field (loginsh, ":,=")) {
+	if (valid_field (loginsh, ":,=") != 0) {
 		fprintf (stderr, _("%s: Invalid entry: %s\n"), Prog, loginsh);
-		closelog ();
-		exit (1);
+		fail_exit (1);
 	}
-	if (!amroot && (!check_shell (loginsh) || access (loginsh, X_OK) != 0)) {
-		fprintf (stderr, _("%s is an invalid shell.\n"), loginsh);
-		closelog ();
-		exit (1);
+	if (   !amroot
+	    && (   is_restricted_shell (loginsh)
+	        || (access (loginsh, X_OK) != 0))) {
+		fprintf (stderr, _("%s: %s is an invalid shell.\n"), Prog, loginsh);
+		fail_exit (1);
 	}
 
 	update_shell (user, loginsh);
 
-	SYSLOG ((LOG_INFO, "changed user `%s' shell to `%s'", user, loginsh));
+	SYSLOG ((LOG_INFO, "changed user '%s' shell to '%s'", user, loginsh));
 
 	nscd_flush_cache ("passwd");
 
-#ifdef USE_PAM
-	pam_end (pamh, PAM_SUCCESS);
-#endif				/* USE_PAM */
-
 	closelog ();
 	exit (E_SUCCESS);
 }
