@@ -2,7 +2,7 @@
  * Copyright (c) 1990 - 1994, Julianne Frances Haugh
  * Copyright (c) 1996 - 2000, Marek Michałkiewicz
  * Copyright (c) 2001 - 2006, Tomasz Kłoczko
- * Copyright (c) 2007 - 2008, Nicolas François
+ * Copyright (c) 2007 - 2009, Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,10 +32,11 @@
 
 #include <config.h>
 
-#ident "$Id: gpasswd.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+#ident "$Id: gpasswd.c 2651 2009-04-11 18:37:59Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
+#include <getopt.h>
 #include <grp.h>
 #include <pwd.h>
 #include <signal.h>
@@ -53,19 +54,21 @@
  * Global variables
  */
 /* The name of this command, as it is invoked */
-static char *Prog;
+char *Prog;
 
 #ifdef SHADOWGRP
 /* Indicate if shadow groups are enabled on the system
  * (/etc/gshadow present) */
-static int is_shadowgrp;
-static int gshadow_locked = 0;
+static bool is_shadowgrp;
 #endif
-static int group_locked = 0;
 
 /* Flags set by options */
-static int
- aflg = 0, Aflg = 0, dflg = 0, Mflg = 0, rflg = 0, Rflg = 0;
+static bool aflg = false;
+static bool Aflg = false;
+static bool dflg = false;
+static bool Mflg = false;
+static bool rflg = false;
+static bool Rflg = false;
 /* The name of the group that is being affected */
 static char *group = NULL;
 /* The name of the user being added (-a) or removed (-d) from group */
@@ -79,7 +82,7 @@ static char *admins = NULL;
 /* The name of the caller */
 static char *myname = NULL;
 /* The UID of the caller */
-static unsigned long bywho = -1;
+static uid_t bywho;
 /* Indicate if gpasswd was called by root */
 #define amroot	(0 == bywho)
 
@@ -91,8 +94,7 @@ static unsigned long bywho = -1;
 /* local function prototypes */
 static void usage (void);
 static RETSIGTYPE catch_signals (int killed);
-static void fail_exit (int status);
-static int check_list (const char *users);
+static bool is_valid_user_list (const char *users);
 static void process_flags (int argc, char **argv);
 static void check_flags (int argc, int opt_index);
 static void open_files (void);
@@ -108,21 +110,44 @@ static void check_perms (const struct group *gr);
 static void update_group (struct group *gr);
 static void change_passwd (struct group *gr);
 #endif
+static void log_gpasswd_failure (const char *suffix);
+static void log_gpasswd_failure_system (unused void *arg);
+static void log_gpasswd_failure_group (unused void *arg);
+#ifdef SHADOWGRP
+static void log_gpasswd_failure_gshadow (unused void *arg);
+#endif
+static void log_gpasswd_success (const char *suffix);
+static void log_gpasswd_success_system (unused void *arg);
+static void log_gpasswd_success_group (unused void *arg);
+#ifdef SHADOWGRP
+static void log_gpasswd_success_gshadow (unused void *arg);
+#endif
 
 /*
  * usage - display usage message
  */
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: %s [-r|-R] group\n"), Prog);
-	fprintf (stderr, _("       %s [-a user] group\n"), Prog);
-	fprintf (stderr, _("       %s [-d user] group\n"), Prog);
-#ifdef SHADOWGRP
 	fprintf (stderr,
-		 _("       %s [-A user,...] [-M user,...] group\n"), Prog);
+	         _("Usage: %s [option] GROUP\n"
+	           "\n"
+	           "Options:\n"
+	           "  -a, --add USER                add USER to GROUP\n"
+	           "  -d, --delete USER             remove USER from GROUP\n"
+	           "  -r, --remove-password         remove the GROUP's password\n"
+	           "  -R, --restrict                restrict access to GROUP to its members\n"
+	           "  -M, --members USER,...        set the list of members of GROUP\n"
+	           "%s\n"
+	           "\n"),
+	         Prog,
+#ifdef SHADOWGRP
+	         _("  -A, --administrators ADMIN,...\n"
+	           "                                set the list of administrators for GROUP\n"
+	           "Except for the -A and -M options, the options cannot be combined.\n")
 #else
-	fprintf (stderr, _("       %s [-M user,...] group\n"), Prog);
+	         _("The options cannot be combined.\n")
 #endif
+	        );
 	exit (E_USAGE);
 }
 
@@ -138,84 +163,64 @@ static RETSIGTYPE catch_signals (int killed)
 {
 	static TERMIO sgtty;
 
-	if (killed) {
+	if (0 != killed) {
 		STTY (0, &sgtty);
 	} else {
 		GTTY (0, &sgtty);
 	}
 
-	if (killed) {
-		putchar ('\n');
-		fflush (stdout);
-		fail_exit (killed);
+	if (0 != killed) {
+		(void) putchar ('\n');
+		(void) fflush (stdout);
+		exit (killed);
 	}
 }
 
 /*
- * fail_exit - undo as much as possible
- */
-static void fail_exit (int status)
-{
-	if (group_locked) {
-		gr_unlock ();
-	}
-#ifdef SHADOWGRP
-	if (gshadow_locked) {
-		sgr_unlock ();
-	}
-#endif
-
-	exit (status);
-}
-
-/*
- * check_list - check a comma-separated list of user names for validity
+ * is_valid_user_list - check a comma-separated list of user names for validity
  *
- *	check_list scans a comma-separated list of user names and checks
- *	that each listed name exists.
+ *	is_valid_user_list scans a comma-separated list of user names and
+ *	checks that each listed name exists is the user database.
  *
- *	It returns 0 on success.
+ *	It returns true if the list of users is valid.
  */
-static int check_list (const char *users)
+static bool is_valid_user_list (const char *users)
 {
-	const char *start, *end;
-	char username[32];
-	int errors = 0;
-	size_t len;
+	char *username, *end;
+	bool is_valid = true;
+	char *tmpusers = xstrdup (users);
 
-	for (start = users; (NULL != start) && ('\0' != *start); start = end) {
-		end = strchr (start, ',');
+	for (username = tmpusers;
+	     (NULL != username) && ('\0' != *username);
+	     username = end) {
+		end = strchr (username, ',');
 		if (NULL != end) {
-			len = end - start;
+			*end = '\0';
 			end++;
-		} else {
-			len = strlen (start);
 		}
 
-		if (len > sizeof (username) - 1) {
-			len = sizeof (username) - 1;
-		}
-		strncpy (username, start, len);
-		username[len] = '\0';
-
 		/*
 		 * This user must exist.
 		 */
 
 		/* local, no need for xgetpwnam */
 		if (getpwnam (username) == NULL) {
-			fprintf (stderr, _("%s: unknown user %s\n"),
+			fprintf (stderr, _("%s: user '%s' does not exist\n"),
 			         Prog, username);
-			errors++;
+			is_valid = false;
 		}
 	}
-	return errors;
+
+	free (tmpusers);
+
+	return is_valid;
 }
 
 static void failure (void)
 {
 	fprintf (stderr, _("%s: Permission denied.\n"), Prog);
-	fail_exit (1);
+	log_gpasswd_failure (": Permission denied");
+	exit (1);
 }
 
 /*
@@ -224,74 +229,63 @@ static void failure (void)
 static void process_flags (int argc, char **argv)
 {
 	int flag;
+	int option_index = 0;
+	static struct option long_options[] = {
+		{"add", required_argument, NULL, 'a'},
+		{"delete", required_argument, NULL, 'd'},
+		{"remove-password", no_argument, NULL, 'r'},
+		{"restrict", no_argument, NULL, 'R'},
+		{"administrators", required_argument, NULL, 'A'},
+		{"members", required_argument, NULL, 'M'},
+		{NULL, 0, NULL, '\0'}
+		};
 
-	while ((flag = getopt (argc, argv, "a:A:d:gM:rR")) != EOF) {
+	while ((flag = getopt_long (argc, argv, "a:A:d:gM:rR", long_options, &option_index)) != -1) {
 		switch (flag) {
 		case 'a':	/* add a user */
+			aflg = true;
 			user = optarg;
 			/* local, no need for xgetpwnam */
 			if (getpwnam (user) == NULL) {
 				fprintf (stderr,
-					 _("%s: unknown user %s\n"), Prog,
-					 user);
-#ifdef WITH_AUDIT
-				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-					      "adding to group", user, -1, 0);
-#endif
-				fail_exit (1);
+				         _("%s: user '%s' does not exist\n"), Prog,
+				         user);
+				exit (1);
 			}
-			aflg++;
 			break;
 #ifdef SHADOWGRP
-		case 'A':
-			if (!amroot) {
-#ifdef WITH_AUDIT
-				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-					      "Listing administrators", NULL,
-					      bywho, 0);
-#endif
-				failure ();
-			}
+		case 'A':	/* set the list of administrators */
 			if (!is_shadowgrp) {
 				fprintf (stderr,
-					 _
-					 ("%s: shadow group passwords required for -A\n"),
-					 Prog);
-				fail_exit (2);
+				         _("%s: shadow group passwords required for -A\n"),
+				         Prog);
+				exit (2);
 			}
 			admins = optarg;
-			if (check_list (admins) != 0) {
-				fail_exit (1);
+			if (!is_valid_user_list (admins)) {
+				exit (1);
 			}
-			Aflg++;
+			Aflg = true;
 			break;
-#endif
+#endif				/* SHADOWGRP */
 		case 'd':	/* delete a user */
-			dflg++;
+			dflg = true;
 			user = optarg;
 			break;
 		case 'g':	/* no-op from normal password */
 			break;
-		case 'M':
-			if (!amroot) {
-#ifdef WITH_AUDIT
-				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-					      "listing members", NULL, bywho,
-					      0);
-#endif
-				failure ();
-			}
+		case 'M':	/* set the list of members */
 			members = optarg;
-			if (check_list (members) != 0) {
-				fail_exit (1);
+			if (!is_valid_user_list (members)) {
+				exit (1);
 			}
-			Mflg++;
+			Mflg = true;
 			break;
 		case 'r':	/* remove group password */
-			rflg++;
+			rflg = true;
 			break;
 		case 'R':	/* restrict group password */
-			Rflg++;
+			Rflg = true;
 			break;
 		default:
 			usage ();
@@ -309,10 +303,26 @@ static void process_flags (int argc, char **argv)
  */
 static void check_flags (int argc, int opt_index)
 {
+	int exclusive = 0;
 	/*
 	 * Make sure exclusive flags are exclusive
 	 */
-	if (aflg + dflg + rflg + Rflg + (Aflg || Mflg) > 1) {
+	if (aflg) {
+		exclusive++;
+	}
+	if (dflg) {
+		exclusive++;
+	}
+	if (rflg) {
+		exclusive++;
+	}
+	if (Rflg) {
+		exclusive++;
+	}
+	if (Aflg || Mflg) {
+		exclusive++;
+	}
+	if (exclusive > 1) {
 		usage ();
 	}
 
@@ -332,52 +342,320 @@ static void check_flags (int argc, int opt_index)
 static void open_files (void)
 {
 	if (gr_lock () == 0) {
-		fprintf (stderr, _("%s: can't get lock\n"), Prog);
-		SYSLOG ((LOG_WARN, "failed to get lock for /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "locking /etc/group", group, -1, 0);
-#endif
-		fail_exit (1);
+		fprintf (stderr,
+		         _("%s: cannot lock %s; try again later.\n"),
+		         Prog, gr_dbname ());
+		exit (1);
 	}
-	group_locked++;
+	add_cleanup (cleanup_unlock_group, NULL);
+
 #ifdef SHADOWGRP
 	if (is_shadowgrp) {
 		if (sgr_lock () == 0) {
 			fprintf (stderr,
-			         _("%s: can't get shadow lock\n"), Prog);
-			SYSLOG ((LOG_WARN, "failed to get lock for /etc/gshadow"));
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "locking /etc/gshadow", group, -1, 0);
-#endif
-			fail_exit (1);
+			         _("%s: cannot lock %s; try again later.\n"),
+			         Prog, sgr_dbname ());
+			exit (1);
 		}
-		gshadow_locked++;
+		add_cleanup (cleanup_unlock_gshadow, NULL);
 	}
-#endif
+#endif				/* SHADOWGRP */
+
+	add_cleanup (log_gpasswd_failure_system, NULL);
+
 	if (gr_open (O_RDWR) == 0) {
-		fprintf (stderr, _("%s: can't open file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot open /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "opening /etc/group", group, -1, 0);
-#endif
-		fail_exit (1);
+		fprintf (stderr,
+		         _("%s: cannot open %s\n"),
+		         Prog, gr_dbname ());
+		SYSLOG ((LOG_WARN, "cannot open %s", gr_dbname ()));
+		exit (1);
 	}
+
 #ifdef SHADOWGRP
-	if (is_shadowgrp && (sgr_open (O_RDWR) == 0)) {
-		fprintf (stderr, _("%s: can't open shadow file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot open /etc/gshadow"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "opening /etc/gshadow", group, -1, 0);
-#endif
-		fail_exit (1);
+	if (is_shadowgrp) {
+		if (sgr_open (O_RDWR) == 0) {
+			fprintf (stderr,
+			         _("%s: cannot open %s\n"),
+			         Prog, sgr_dbname ());
+			SYSLOG ((LOG_WARN, "cannot open %s", sgr_dbname ()));
+			exit (1);
+		}
+		add_cleanup (log_gpasswd_failure_gshadow, NULL);
 	}
-#endif
+#endif				/* SHADOWGRP */
+
+	add_cleanup (log_gpasswd_failure_group, NULL);
+	del_cleanup (log_gpasswd_failure_system);
 }
 
+static void log_gpasswd_failure (const char *suffix)
+{
+#ifdef WITH_AUDIT
+	char buf[1024];
+#endif
+	if (aflg) {
+		SYSLOG ((LOG_ERR,
+		         "%s failed to add user %s to group %s%s",
+		         myname, user, group, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "%s failed to add user %s to group %s%s",
+		          myname, user, group, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_ACCT, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_FAILURE);
+#endif
+	} else if (dflg) {
+		SYSLOG ((LOG_ERR,
+		         "%s failed to remove user %s from group %s%s",
+		         myname, user, group, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "%s failed to remove user %s from group %s%s",
+		          myname, user, group, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_ACCT, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_FAILURE);
+#endif
+	} else if (rflg) {
+		SYSLOG ((LOG_ERR,
+		         "%s failed to remove password of group %s%s",
+		         myname, group, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "%s failed to remove password of group %s%s",
+		          myname, group, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_FAILURE);
+#endif
+	} else if (Rflg) {
+		SYSLOG ((LOG_ERR,
+		         "%s failed to restrict access to group %s%s",
+		         myname, group, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "%s failed to restrict access to group %s%s",
+		          myname, group, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_FAILURE);
+#endif
+	} else if (Aflg || Mflg) {
+#ifdef SHADOWGRP
+		if (Aflg) {
+			SYSLOG ((LOG_ERR,
+			         "%s failed to set the administrators of group %s to %s%s",
+			         myname, group, admins, suffix));
+#ifdef WITH_AUDIT
+			snprintf (buf, 1023,
+			          "%s failed to set the administrators of group %s to %s%s",
+			          myname, group, admins, suffix);
+			buf[1023] = '\0';
+			audit_logger (AUDIT_USER_ACCT, Prog,
+			              buf,
+			              group, AUDIT_NO_ID,
+			              SHADOW_AUDIT_FAILURE);
+#endif
+		}
+#endif				/* SHADOWGRP */
+		if (Mflg) {
+			SYSLOG ((LOG_ERR,
+			         "%s failed to set the members of group %s to %s%s",
+			         myname, group, members, suffix));
+#ifdef WITH_AUDIT
+			snprintf (buf, 1023,
+			          "%s failed to set the members of group %s to %s%s",
+			          myname, group, members, suffix);
+			buf[1023] = '\0';
+			audit_logger (AUDIT_USER_ACCT, Prog,
+			              buf,
+			              group, AUDIT_NO_ID,
+			              SHADOW_AUDIT_FAILURE);
+#endif
+		}
+	} else {
+		SYSLOG ((LOG_ERR,
+		         "%s failed to change password of group %s%s",
+		         myname, group, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "%s failed to change password of group %s%s",
+		          myname, group, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_FAILURE);
+#endif
+	}
+}
+
+static void log_gpasswd_failure_system (unused void *arg)
+{
+	log_gpasswd_failure ("");
+}
+
+static void log_gpasswd_failure_group (unused void *arg)
+{
+	char buf[1024];
+	snprintf (buf, 1023, " in %s", gr_dbname ());
+	buf[1023] = '\0';
+	log_gpasswd_failure (buf);
+}
+
+#ifdef SHADOWGRP
+static void log_gpasswd_failure_gshadow (unused void *arg)
+{
+	char buf[1024];
+	snprintf (buf, 1023, " in %s", sgr_dbname ());
+	buf[1023] = '\0';
+	log_gpasswd_failure (buf);
+}
+#endif				/* SHADOWGRP */
+
+static void log_gpasswd_success (const char *suffix)
+{
+#ifdef WITH_AUDIT
+	char buf[1024];
+#endif
+	if (aflg) {
+		SYSLOG ((LOG_INFO,
+		         "user %s added by %s to group %s%s",
+		         user, myname, group, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "user %s added by %s to group %s%s",
+		          user, myname, group, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_ACCT, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_SUCCESS);
+#endif
+	} else if (dflg) {
+		SYSLOG ((LOG_INFO,
+		         "user %s removed by %s from group %s%s",
+		         user, myname, group, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "user %s removed by %s from group %s%s",
+		          user, myname, group, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_ACCT, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_SUCCESS);
+#endif
+	} else if (rflg) {
+		SYSLOG ((LOG_INFO,
+		         "password of group %s removed by %s%s",
+		         group, myname, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "password of group %s removed by %s%s",
+		          group, myname, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_SUCCESS);
+#endif
+	} else if (Rflg) {
+		SYSLOG ((LOG_INFO,
+		         "access to group %s restricted by %s%s",
+		         group, myname, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "access to group %s restricted by %s%s",
+		          group, myname, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_SUCCESS);
+#endif
+	} else if (Aflg || Mflg) {
+#ifdef SHADOWGRP
+		if (Aflg) {
+			SYSLOG ((LOG_INFO,
+			         "administrators of group %s set by %s to %s%s",
+			         group, myname, admins, suffix));
+#ifdef WITH_AUDIT
+			snprintf (buf, 1023,
+			          "administrators of group %s set by %s to %s%s",
+			          group, myname, admins, suffix);
+			buf[1023] = '\0';
+			audit_logger (AUDIT_USER_ACCT, Prog,
+			              buf,
+			              group, AUDIT_NO_ID,
+			              SHADOW_AUDIT_SUCCESS);
+#endif
+		}
+#endif				/* SHADOWGRP */
+		if (Mflg) {
+			SYSLOG ((LOG_INFO,
+			         "members of group %s set by %s to %s%s",
+			         group, myname, members, suffix));
+#ifdef WITH_AUDIT
+			snprintf (buf, 1023,
+			          "members of group %s set by %s to %s%s",
+			          group, myname, members, suffix);
+			buf[1023] = '\0';
+			audit_logger (AUDIT_USER_ACCT, Prog,
+			              buf,
+			              group, AUDIT_NO_ID,
+			              SHADOW_AUDIT_SUCCESS);
+#endif
+		}
+	} else {
+		SYSLOG ((LOG_INFO,
+		         "password of group %s changed by %s%s",
+		         group, myname, suffix));
+#ifdef WITH_AUDIT
+		snprintf (buf, 1023,
+		          "password of group %s changed by %s%s",
+		          group, myname, suffix);
+		buf[1023] = '\0';
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              buf,
+		              group, AUDIT_NO_ID,
+		              SHADOW_AUDIT_SUCCESS);
+#endif
+	}
+}
+
+static void log_gpasswd_success_system (unused void *arg)
+{
+	log_gpasswd_success ("");
+}
+
+static void log_gpasswd_success_group (unused void *arg)
+{
+	char buf[1024];
+	snprintf (buf, 1023, " in %s", gr_dbname ());
+	buf[1023] = '\0';
+	log_gpasswd_success (buf);
+}
+
+#ifdef SHADOWGRP
+static void log_gpasswd_success_gshadow (unused void *arg)
+{
+	char buf[1024];
+	snprintf (buf, 1023, " in %s", sgr_dbname ());
+	buf[1023] = '\0';
+	log_gpasswd_success (buf);
+}
+#endif				/* SHADOWGRP */
+
 /*
  * close_files - close and unlock the group databases
  *
@@ -388,39 +666,40 @@ static void open_files (void)
 static void close_files (void)
 {
 	if (gr_close () == 0) {
-		fprintf (stderr, _("%s: can't re-write file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot re-write /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "rewriting /etc/group", group, -1, 0);
-#endif
-		fail_exit (1);
-	}
-#ifdef SHADOWGRP
-	if (is_shadowgrp && (sgr_close () == 0)) {
-		fprintf (stderr, _("%s: can't re-write shadow file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot re-write /etc/gshadow"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "rewriting /etc/gshadow", group, -1, 0);
-#endif
-		fail_exit (1);
-	}
-	if (is_shadowgrp) {
-		/* TODO: same logging as in open_files & for /etc/group */
-		sgr_unlock ();
-		gshadow_locked--;
-	}
-#endif
-	if (gr_unlock () == 0) {
-		fprintf (stderr, _("%s: can't unlock file\n"), Prog);
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "unlocking group file", group, -1, 0);
-#endif
+		fprintf (stderr,
+		         _("%s: failure while writing changes to %s\n"),
+		         Prog, gr_dbname ());
 		exit (1);
 	}
-	group_locked--;
+	add_cleanup (log_gpasswd_success_group, NULL);
+	del_cleanup (log_gpasswd_failure_group);
+
+	cleanup_unlock_group (NULL);
+	del_cleanup (cleanup_unlock_group);
+
+#ifdef SHADOWGRP
+	if (is_shadowgrp) {
+		if (sgr_close () == 0) {
+			fprintf (stderr,
+			         _("%s: failure while writing changes to %s\n"),
+			         Prog, sgr_dbname ());
+			exit (1);
+		}
+		add_cleanup (log_gpasswd_success_gshadow, NULL);
+		del_cleanup (log_gpasswd_failure_gshadow);
+
+		cleanup_unlock_gshadow (NULL);
+		del_cleanup (cleanup_unlock_gshadow);
+	}
+#endif				/* SHADOWGRP */
+
+	log_gpasswd_success_system (NULL);
+	del_cleanup (log_gpasswd_success_group);
+#ifdef SHADOWGRP
+	if (is_shadowgrp) {
+		del_cleanup (log_gpasswd_success_gshadow);
+	}
+#endif
 }
 
 /*
@@ -435,6 +714,13 @@ static void check_perms (const struct group *gr, const struct sgrp *sg)
 static void check_perms (const struct group *gr)
 #endif
 {
+	/*
+	 * Only root can use the -M and -A options.
+	 */
+	if (!amroot && (Aflg || Mflg)) {
+		failure ();
+	}
+
 #ifdef SHADOWGRP
 	if (is_shadowgrp) {
 		/*
@@ -445,14 +731,10 @@ static void check_perms (const struct group *gr)
 		 * the root user can.
 		 */
 		if (!amroot && !is_on_list (sg->sg_adm, myname)) {
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "modify group", group, -1, 0);
-#endif
 			failure ();
 		}
 	} else
-#endif				/* ! SHADOWGRP */
+#endif				/* SHADOWGRP */
 	{
 #ifdef FIRST_MEMBER_IS_ADMIN
 		/*
@@ -471,27 +753,15 @@ static void check_perms (const struct group *gr)
 		 */
 		if (!amroot) {
 			if (gr->gr_mem[0] == (char *) 0) {
-#ifdef WITH_AUDIT
-				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-				              "modifying group", group, -1, 0);
-#endif
 				failure ();
 			}
 
 			if (strcmp (gr->gr_mem[0], myname) != 0) {
-#ifdef WITH_AUDIT
-				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-				              "modifying group", myname, -1, 0);
-#endif
 				failure ();
 			}
 		}
 #else				/* ! FIRST_MEMBER_IS_ADMIN */
 		if (!amroot) {
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "modifying group", group, -1, 0);
-#endif
 			failure ();
 		}
 #endif
@@ -508,25 +778,19 @@ static void update_group (struct group *gr)
 #endif
 {
 	if (gr_update (gr) == 0) {
-		fprintf (stderr, _("%s: can't update entry\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot update /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "updating /etc/group", group, -1, 0);
-#endif
-		fail_exit (1);
+		fprintf (stderr,
+		         _("%s: failed to prepare the new %s entry '%s'\n"),
+		         Prog, gr_dbname (), gr->gr_name);
+		exit (1);
 	}
 #ifdef SHADOWGRP
 	if (is_shadowgrp && (sgr_update (sg) == 0)) {
-		fprintf (stderr, _("%s: can't update shadow entry\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot update /etc/gshadow"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "updating /etc/gshadow", group, -1, 0);
-#endif
-		fail_exit (1);
+		fprintf (stderr,
+		         _("%s: failed to prepare the new %s entry '%s'\n"),
+		         Prog, sgr_dbname (), sg->sg_name);
+		exit (1);
 	}
-#endif
+#endif				/* SHADOWGRP */
 }
 
 /*
@@ -544,26 +808,22 @@ static void get_group (struct group *gr)
 #endif
 {
 	struct group const*tmpgr = NULL;
+#ifdef SHADOWGRP
 	struct sgrp const*tmpsg = NULL;
+#endif
 
 	if (gr_open (O_RDONLY) == 0) {
-		fprintf (stderr, _("%s: can't open file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot open /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "opening /etc/group", group, -1, 0);
-#endif
-		fail_exit (1);
+		fprintf (stderr, _("%s: cannot open %s\n"), Prog, gr_dbname ());
+		SYSLOG ((LOG_WARN, "cannot open %s", gr_dbname ()));
+		exit (1);
 	}
 
 	tmpgr = gr_locate (group);
 	if (NULL == tmpgr) {
-		fprintf (stderr, _("unknown group: %s\n"), group);
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "group lookup", group, -1, 0);
-#endif
-		failure ();
+		fprintf (stderr,
+		         _("%s: group '%s' does not exist in %s\n"),
+		         Prog, group, gr_dbname ());
+		exit (1);
 	}
 
 	*gr = *tmpgr;
@@ -572,26 +832,23 @@ static void get_group (struct group *gr)
 	gr->gr_mem = dup_list (tmpgr->gr_mem);
 
 	if (gr_close () == 0) {
-		fprintf (stderr, _("%s: can't close file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot close /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "closing /etc/group", group, -1, 0);
-#endif
-		fail_exit (1);
+		fprintf (stderr,
+		         _("%s: failure while closing read-only %s\n"),
+		         Prog, gr_dbname ());
+		SYSLOG ((LOG_ERR,
+		         "failure while closing read-only %s",
+		         gr_dbname ()));
+		exit (1);
 	}
 
 #ifdef SHADOWGRP
 	if (is_shadowgrp) {
 		if (sgr_open (O_RDONLY) == 0) {
 			fprintf (stderr,
-			         _("%s: can't open shadow file\n"), Prog);
-			SYSLOG ((LOG_WARN, "cannot open /etc/gshadow"));
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "opening /etc/gshadow", group, -1, 0);
-#endif
-			fail_exit (1);
+			         _("%s: cannot open %s\n"),
+			         Prog, sgr_dbname ());
+			SYSLOG ((LOG_WARN, "cannot open %s", sgr_dbname ()));
+			exit (1);
 		}
 		tmpsg = sgr_locate (group);
 		if (NULL != tmpsg) {
@@ -622,13 +879,12 @@ static void get_group (struct group *gr)
 		}
 		if (sgr_close () == 0) {
 			fprintf (stderr,
-			         _("%s: can't close shadow file\n"), Prog);
-			SYSLOG ((LOG_WARN, "cannot close /etc/gshadow"));
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "closing /etc/gshadow", group, -1, 0);
-#endif
-			fail_exit (1);
+			         _("%s: failure while closing read-only %s\n"),
+			         Prog, sgr_dbname ());
+			SYSLOG ((LOG_ERR,
+			         "failure while closing read-only %s",
+			         sgr_dbname ()));
+			exit (1);
 		}
 	}
 #endif				/* SHADOWGRP */
@@ -663,14 +919,14 @@ static void change_passwd (struct group *gr)
 	for (retries = 0; retries < RETRIES; retries++) {
 		cp = getpass (_("New Password: "));
 		if (NULL == cp) {
-			fail_exit (1);
+			exit (1);
 		}
 
 		STRFCPY (pass, cp);
 		strzero (cp);
 		cp = getpass (_("Re-enter new password: "));
 		if (NULL == cp) {
-			fail_exit (1);
+			exit (1);
 		}
 
 		if (strcmp (pass, cp) == 0) {
@@ -683,16 +939,12 @@ static void change_passwd (struct group *gr)
 
 		if (retries + 1 < RETRIES) {
 			puts (_("They don't match; try again"));
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "changing password", group, -1, 0);
-#endif
 		}
 	}
 
 	if (retries == RETRIES) {
 		fprintf (stderr, _("%s: Try again later\n"), Prog);
-		fail_exit (1);
+		exit (1);
 	}
 
 	cp = pw_encrypt (pass, crypt_make_salt (NULL, NULL));
@@ -705,23 +957,10 @@ static void change_passwd (struct group *gr)
 	{
 		gr->gr_passwd = cp;
 	}
-#ifdef WITH_AUDIT
-	audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-	              "changing password", group, -1, 1);
-#endif
-	SYSLOG ((LOG_INFO, "change the password for group %s by %s", group,
-	        myname));
 }
 
 /*
  * gpasswd - administer the /etc/group file
- *
- *	-a user		add user to the named group
- *	-d user		remove user from the named group
- *	-r		remove password from the named group
- *	-R		restrict access to the named group
- *	-A user,...	make list of users the administrative users
- *	-M user,...	make list of users the group members
  */
 int main (int argc, char **argv)
 {
@@ -736,15 +975,15 @@ int main (int argc, char **argv)
 #endif
 
 	sanitize_env ();
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
+	(void) setlocale (LC_ALL, "");
+	(void) bindtextdomain (PACKAGE, LOCALEDIR);
+	(void) textdomain (PACKAGE);
 
 	/*
 	 * Make a note of whether or not this command was invoked by root.
 	 * This will be used to bypass certain checks later on. Also, set
 	 * the real user ID to match the effective user ID. This will
-	 * prevent the invoker from issuing signals which would interfer
+	 * prevent the invoker from issuing signals which would interfere
 	 * with this command.
 	 */
 	bywho = getuid ();
@@ -758,27 +997,35 @@ int main (int argc, char **argv)
 	is_shadowgrp = sgr_file_present ();
 #endif
 
-	/* Parse the options */
-	process_flags (argc, argv);
-
 	/*
 	 * Determine the name of the user that invoked this command. This
 	 * is really hit or miss because there are so many ways that command
 	 * can be executed and so many ways to trip up the routines that
 	 * report the user name.
 	 */
-
 	pw = get_my_pwent ();
 	if (NULL == pw) {
-		fputs (_("Who are you?\n"), stderr);
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "user lookup", NULL,
-		              bywho, 0);
-#endif
-		failure ();
+		fprintf (stderr, _("%s: Cannot determine your user name.\n"),
+		         Prog);
+		SYSLOG ((LOG_WARN,
+		         "Cannot determine the user name of the caller (UID %lu)",
+		         (unsigned long) getuid ()));
+		exit (1);
 	}
 	myname = xstrdup (pw->pw_name);
 
+	/*
+	 * Register an exit function to warn for any inconsistency that we
+	 * could create.
+	 */
+	if (atexit (do_cleanups) != 0) {
+		fprintf(stderr, "%s: cannot set exit function\n", Prog);
+		exit(EXIT_FAILURE);
+	}
+
+	/* Parse the options */
+	process_flags (argc, argv);
+
 	/*
 	 * Replicate the group so it can be modified later on.
 	 */
@@ -806,12 +1053,6 @@ int main (int argc, char **argv)
 #ifdef SHADOWGRP
 		sgent.sg_passwd = "";	/* XXX warning: const */
 #endif
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "deleting group password", group, -1, 1);
-#endif
-		SYSLOG ((LOG_INFO, "remove password from group %s by %s",
-		         group, myname));
 		goto output;
 	} else if (Rflg) {
 		/*
@@ -822,12 +1063,6 @@ int main (int argc, char **argv)
 #ifdef SHADOWGRP
 		sgent.sg_passwd = "!";	/* XXX warning: const */
 #endif
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "restrict access to group", group, -1, 1);
-#endif
-		SYSLOG ((LOG_INFO, "restrict access to group %s by %s",
-			 group, myname));
 		goto output;
 	}
 
@@ -843,12 +1078,6 @@ int main (int argc, char **argv)
 			sgent.sg_mem = add_list (sgent.sg_mem, user);
 		}
 #endif
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding group member",
-		              user, -1, 1);
-#endif
-		SYSLOG ((LOG_INFO, "add member %s to group %s by %s", user,
-		         group, myname));
 		goto output;
 	}
 
@@ -857,58 +1086,43 @@ int main (int argc, char **argv)
 	 * one, except the routine is different.
 	 */
 	if (dflg) {
-		int removed = 0;
+		bool removed = false;
 
 		printf (_("Removing user %s from group %s\n"), user, group);
 
 		if (is_on_list (grent.gr_mem, user)) {
-			removed = 1;
+			removed = true;
 			grent.gr_mem = del_list (grent.gr_mem, user);
 		}
 #ifdef SHADOWGRP
 		if (is_shadowgrp) {
 			if (is_on_list (sgent.sg_mem, user)) {
-				removed = 1;
+				removed = true;
 				sgent.sg_mem = del_list (sgent.sg_mem, user);
 			}
 		}
 #endif
 		if (!removed) {
-			fprintf (stderr, _("%s: unknown member %s\n"),
-			         Prog, user);
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			              "deleting member", user, -1, 0);
-#endif
-			fail_exit (1);
+			fprintf (stderr,
+			         _("%s: user '%s' is not a member of '%s'\n"),
+			         Prog, user, group);
+			exit (1);
 		}
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting member",
-		              user, -1, 1);
-#endif
-		SYSLOG ((LOG_INFO, "remove member %s from group %s by %s",
-		         user, group, myname));
 		goto output;
 	}
 #ifdef SHADOWGRP
 	/*
-	 * Replacing the entire list of administators is simple. Check the
+	 * Replacing the entire list of administrators is simple. Check the
 	 * list to make sure everyone is a real user. Then slap the new list
 	 * in place.
 	 */
 	if (Aflg) {
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "setting group admin",
-		              group, -1, 1);
-#endif
-		SYSLOG ((LOG_INFO, "set administrators of %s to %s",
-		         group, admins));
 		sgent.sg_adm = comma_to_list (admins);
 		if (!Mflg) {
 			goto output;
 		}
 	}
-#endif
+#endif				/* SHADOWGRP */
 
 	/*
 	 * Replacing the entire list of members is simple. Check the list to
@@ -916,11 +1130,6 @@ int main (int argc, char **argv)
 	 * place.
 	 */
 	if (Mflg) {
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-		              "setting group members", group, -1, 1);
-#endif
-		SYSLOG ((LOG_INFO, "set members of %s to %s", group, members));
 #ifdef SHADOWGRP
 		sgent.sg_mem = comma_to_list (members);
 #endif
@@ -935,21 +1144,17 @@ int main (int argc, char **argv)
 	 */
 	if ((isatty (0) == 0) || (isatty (1) == 0)) {
 		fprintf (stderr, _("%s: Not a tty\n"), Prog);
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing password",
-		              group, -1, 0);
-#endif
-		fail_exit (1);
+		exit (1);
 	}
 
 	catch_signals (0);	/* save tty modes */
 
-	signal (SIGHUP, catch_signals);
-	signal (SIGINT, catch_signals);
-	signal (SIGQUIT, catch_signals);
-	signal (SIGTERM, catch_signals);
+	(void) signal (SIGHUP, catch_signals);
+	(void) signal (SIGINT, catch_signals);
+	(void) signal (SIGQUIT, catch_signals);
+	(void) signal (SIGTERM, catch_signals);
 #ifdef SIGTSTP
-	signal (SIGTSTP, catch_signals);
+	(void) signal (SIGTSTP, catch_signals);
 #endif
 
 	/* Prompt for the new password */
@@ -969,12 +1174,8 @@ int main (int argc, char **argv)
 	if (setuid (0) != 0) {
 		fputs (_("Cannot change ID to root.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing id to root",
-		              group, -1, 0);
-#endif
 		closelog ();
-		fail_exit (1);
+		exit (1);
 	}
 	pwd_init ();
 
@@ -992,3 +1193,4 @@ int main (int argc, char **argv)
 
 	exit (E_SUCCESS);
 }
+
