@@ -40,7 +40,7 @@
 
 #include <config.h>
 
-#ident "$Id: id.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+#ident "$Id: id.c 2167 2008-06-13 21:31:23Z nekral-guest $"
 
 #include <grp.h>
 #include <pwd.h>
@@ -53,9 +53,9 @@ static void usage (void);
 static void usage (void)
 {
 #ifdef HAVE_GETGROUPS
-	fputs (_("Usage: id [-a]\n"), stderr);
+	(void) fputs (_("Usage: id [-a]\n"), stderr);
 #else
-	fputs (_("Usage: id\n"), stderr);
+	(void) fputs (_("Usage: id\n"), stderr);
 #endif
 	exit (1);
 }
@@ -77,14 +77,14 @@ static void usage (void)
 #ifdef HAVE_GETGROUPS
 	GETGROUPS_T *groups;
 	int ngroups;
-	int aflg = 0;
+	bool aflg = 0;
 #endif
 	struct passwd *pw;
 	struct group *gr;
 
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
+	(void) setlocale (LC_ALL, "");
+	(void) bindtextdomain (PACKAGE, LOCALEDIR);
+	(void) textdomain (PACKAGE);
 
 	/*
 	 * Dynamically get the maximum number of groups from system, instead
@@ -94,21 +94,23 @@ static void usage (void)
 	 */
 	sys_ngroups = sysconf (_SC_NGROUPS_MAX);
 #ifdef HAVE_GETGROUPS
-	groups = (GETGROUPS_T *) malloc (sys_ngroups * sizeof (GETGROUPS_T));
+	groups = (GETGROUPS_T *) malloc (sizeof (GETGROUPS_T) * sys_ngroups);
 	/*
 	 * See if the -a flag has been given to print out the concurrent
 	 * group set.
 	 */
 
 	if (argc > 1) {
-		if (argc > 2 || strcmp (argv[1], "-a"))
+		if ((argc > 2) || (strcmp (argv[1], "-a") != 0)) {
 			usage ();
-		else
-			aflg = 1;
+		} else {
+			aflg = true;
+		}
 	}
 #else
-	if (argc > 1)
+	if (argc > 1) {
 		usage ();
+	}
 #endif
 
 	ruid = getuid ();
@@ -122,16 +124,20 @@ static void usage (void)
 	 */
 
 	pw = getpwuid (ruid); /* local, no need for xgetpwuid */
-	if (pw)
-		printf ("UID=%u(%s)", ruid, pw->pw_name);
-	else
-		printf ("UID=%u", ruid);
+	if (NULL != pw) {
+		(void) printf ("UID=%lu(%s)",
+		               (unsigned long) ruid, pw->pw_name);
+	} else {
+		(void) printf ("UID=%lu", (unsigned long) ruid);
+	}
 
 	gr = getgrgid (rgid);; /* local, no need for xgetgrgid */
-	if (gr)
-		printf (" GID=%u(%s)", rgid, gr->gr_name);
-	else
-		printf (" GID=%u", rgid);
+	if (NULL != gr) {
+		(void) printf (" GID=%lu(%s)",
+		               (unsigned long) rgid, gr->gr_name);
+	} else {
+		(void) printf (" GID=%lu", (unsigned long) rgid);
+	}
 
 	/*
 	 * Print out the effective user ID and group ID if they are
@@ -140,17 +146,21 @@ static void usage (void)
 
 	if (ruid != euid) {
 		pw = getpwuid (euid); /* local, no need for xgetpwuid */
-		if (pw)
-			printf (" EUID=%u(%s)", euid, pw->pw_name);
-		else
-			printf (" EUID=%u", euid);
+		if (NULL != pw) {
+			(void) printf (" EUID=%lu(%s)",
+			               (unsigned long) euid, pw->pw_name);
+		} else {
+			(void) printf (" EUID=%lu", (unsigned long) euid);
+		}
 	}
 	if (rgid != egid) {
 		gr = getgrgid (egid); /* local, no need for xgetgrgid */
-		if (gr)
-			printf (" EGID=%u(%s)", egid, gr->gr_name);
-		else
-			printf (" EGID=%u", egid);
+		if (NULL != gr) {
+			(void) printf (" EGID=%lu(%s)",
+			               (unsigned long) egid, gr->gr_name);
+		} else {
+			(void) printf (" EGID=%lu", (unsigned long) egid);
+		}
 	}
 #ifdef HAVE_GETGROUPS
 	/*
@@ -167,17 +177,21 @@ static void usage (void)
 		 * where "###" is a numerical value and "aaa" is the
 		 * corresponding name for each respective numerical value.
 		 */
-		puts (_(" groups="));
+		(void) puts (_(" groups="));
 		for (i = 0; i < ngroups; i++) {
-			if (i)
-				putchar (',');
+			if (0 != i)
+				(void) putchar (',');
 
 			/* local, no need for xgetgrgid */
 			gr = getgrgid (groups[i]);
-			if (gr)
-				printf ("%u(%s)", groups[i], gr->gr_name);
-			else
-				printf ("%u", groups[i]);
+			if (NULL != gr) {
+				(void) printf ("%lu(%s)",
+				               (unsigned long) groups[i],
+				               gr->gr_name);
+			} else {
+				(void) printf ("%lu",
+				               (unsigned long) groups[i]);
+			}
 		}
 	}
 	free (groups);
@@ -186,7 +200,7 @@ static void usage (void)
 	/*
 	 * Finish off the line.
 	 */
-	putchar ('\n');
+	(void) putchar ('\n');
 	exit (0);
 	/* NOT REACHED */
 }
