@@ -32,7 +32,7 @@
 
 #include <config.h>
 
-#ident "$Id: login.c 2664 2009-04-12 00:17:36Z nekral-guest $"
+#ident "$Id: login.c 2690 2009-04-15 17:50:17Z nekral-guest $"
 
 #include <errno.h>
 #include <grp.h>
@@ -682,7 +682,7 @@ int main (int argc, char **argv)
 		   set it to NULL */
 		retcode = pam_get_item (pamh, PAM_USER, (const void **)ptr_pam_user);
 		PAM_FAIL_CHECK;
-		if (pam_user[0] == '\0') {
+		if ((NULL != pam_user) && ('\0' == pam_user[0])) {
 			retcode = pam_set_item (pamh, PAM_USER, NULL);
 			PAM_FAIL_CHECK;
 		}
