@@ -2,7 +2,7 @@
  * Copyright (c) 1989 - 1994, Julianne Frances Haugh
  * Copyright (c) 1996 - 2000, Marek Michałkiewicz
  * Copyright (c) 2001 - 2006, Tomasz Kłoczko
- * Copyright (c) 2007 - 2008, Nicolas François
+ * Copyright (c) 2007 - 2009, Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,7 @@
 
 #include <config.h>
 
-#ident "$Id: passwd.c 2003 2008-05-24 13:08:58Z nekral-guest $"
+#ident "$Id: passwd.c 2612 2009-04-10 22:34:23Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -68,37 +68,43 @@
 /*
  * Global variables
  */
+char *Prog;			/* Program name */
+
 static char *name;		/* The name of user whose password is being changed */
 static char *myname;		/* The current user's name */
-static char *Prog;		/* Program name */
-static int amroot;		/* The real UID was 0 */
+static bool amroot;		/* The caller's real UID was 0 */
 
-static int
- aflg = 0,			/* -a - show status for all users */
-    dflg = 0,			/* -d - delete password */
-    eflg = 0,			/* -e - force password change */
-    iflg = 0,			/* -i - set inactive days */
-    kflg = 0,			/* -k - change only if expired */
-    lflg = 0,			/* -l - lock account */
-    nflg = 0,			/* -n - set minimum days */
-    qflg = 0,			/* -q - quiet mode */
-    Sflg = 0,			/* -S - show password status */
-    uflg = 0,			/* -u - unlock account */
-    wflg = 0,			/* -w - set warning days */
-    xflg = 0;			/* -x - set maximum days */
+static bool
+    aflg = false,			/* -a - show status for all users */
+    dflg = false,			/* -d - delete password */
+    eflg = false,			/* -e - force password change */
+    iflg = false,			/* -i - set inactive days */
+    kflg = false,			/* -k - change only if expired */
+    lflg = false,			/* -l - lock the user's password */
+    nflg = false,			/* -n - set minimum days */
+    qflg = false,			/* -q - quiet mode */
+    Sflg = false,			/* -S - show password status */
+    uflg = false,			/* -u - unlock the user's password */
+    wflg = false,			/* -w - set warning days */
+    xflg = false;			/* -x - set maximum days */
 
 /*
  * set to 1 if there are any flags which require root privileges,
  * and require username to be specified
  */
-static int anyflag = 0;
+static bool anyflag = false;
 
 static long age_min = 0;	/* Minimum days before change   */
 static long age_max = 0;	/* Maximum days until change     */
 static long warn = 0;		/* Warning days before change   */
 static long inact = 0;		/* Days without change before locked */
 
-static int do_update_age = 0;
+#ifndef USE_PAM
+static bool do_update_age = false;
+#endif
+
+static bool pw_locked = false;
+static bool spw_locked = false;
 
 #ifndef USE_PAM
 /*
@@ -115,7 +121,7 @@ static int do_update_age = 0;
  *   total	161
  */
 static char crypt_passwd[256];
-static int do_update_pwd = 0;
+static bool do_update_pwd = false;
 #endif
 
 /*
@@ -141,7 +147,6 @@ static char *update_crypt_pw (char *);
 static void update_noshadow (void);
 
 static void update_shadow (void);
-static long getnumber (const char *);
 #ifdef WITH_SELINUX
 static int check_selinux_access (const char *changed_user,
                                  uid_t changed_uid,
@@ -163,15 +168,15 @@ static void usage (int status)
 	         "  -k, --keep-tokens             change password only if expired\n"
 	         "  -i, --inactive INACTIVE       set password inactive after expiration\n"
 	         "                                to INACTIVE\n"
-	         "  -l, --lock                    lock the named account\n"
+	         "  -l, --lock                    lock the password of the named account\n"
 	         "  -n, --mindays MIN_DAYS        set minimum number of days before password\n"
 	         "                                change to MIN_DAYS\n"
 	         "  -q, --quiet                   quiet mode\n"
 	         "  -r, --repository REPOSITORY   change password in REPOSITORY repository\n"
 	         "  -S, --status                  report password status on the named account\n"
-	         "  -u, --unlock                  unlock the named account\n"
+	         "  -u, --unlock                  unlock the password of the named account\n"
 	         "  -w, --warndays WARN_DAYS      set expiration warning days to WARN_DAYS\n"
-	         "  -x, --maxdays MAX_DAYS        set maximim number of days before password\n"
+	         "  -x, --maxdays MAX_DAYS        set maximum number of days before password\n"
 	         "                                change to MAX_DAYS\n"
 	         "\n"), stderr);
 	exit (status);
@@ -192,7 +197,7 @@ static int reuse (const char *pass, const struct passwd *pw)
 
 	reason = FascistHistory (pass, pw->pw_uid);
 #endif
-	if (reason) {
+	if (NULL != reason) {
 		printf (_("Bad password: %s.  "), reason);
 		return 1;
 	}
@@ -226,8 +231,10 @@ static int new_password (const struct passwd *pw)
 	 */
 
 	if (!amroot && crypt_passwd[0]) {
-		if (!(clear = getpass (_("Old password: "))))
+		clear = getpass (_("Old password: "));
+		if (NULL == clear) {
 			return -1;
+		}
 
 		cipher = pw_encrypt (clear, crypt_passwd);
 		if (strcmp (cipher, crypt_passwd) != 0) {
@@ -253,18 +260,20 @@ static int new_password (const struct passwd *pw)
 	 * for initial login passwords.
 	 */
 	if ((method = getdef_str ("ENCRYPT_METHOD")) == NULL) {
-		if (!getdef_bool ("MD5_CRYPT_ENAB"))
+		if (!getdef_bool ("MD5_CRYPT_ENAB")) {
 			pass_max_len = getdef_num ("PASS_MAX_LEN", 8);
+		}
 	} else {
-		if (   !strcmp (method, "MD5")
+		if (   (strcmp (method, "MD5")    == 0)
 #ifdef USE_SHA_CRYPT
-		    || !strcmp (method, "SHA256")
-		    || !strcmp (method, "SHA512")
+		    || (strcmp (method, "SHA256") == 0)
+		    || (strcmp (method, "SHA512") == 0)
 #endif
-		    )
+		    ) {
 			pass_max_len = -1;
-		else
+		} else {
 			pass_max_len = getdef_num ("PASS_MAX_LEN", 8);
+		}
 	}
 	if (!qflg) {
 		if (pass_max_len == -1) {
@@ -282,12 +291,14 @@ static int new_password (const struct passwd *pw)
 
 	warned = 0;
 	for (i = getdef_num ("PASS_CHANGE_TRIES", 5); i > 0; i--) {
-		if (!(cp = getpass (_("New password: ")))) {
+		cp = getpass (_("New password: "));
+		if (NULL == cp) {
 			memzero (orig, sizeof orig);
 			return -1;
 		}
-		if (warned && strcmp (pass, cp) != 0)
+		if (warned && (strcmp (pass, cp) != 0)) {
 			warned = 0;
+		}
 		STRFCPY (pass, cp);
 		strzero (cp);
 
@@ -307,13 +318,14 @@ static int new_password (const struct passwd *pw)
 			warned++;
 			continue;
 		}
-		if (!(cp = getpass (_("Re-enter new password: ")))) {
+		cp = getpass (_("Re-enter new password: "));
+		if (NULL == cp) {
 			memzero (orig, sizeof orig);
 			return -1;
 		}
-		if (strcmp (cp, pass))
+		if (strcmp (cp, pass) != 0) {
 			fputs (_("They don't match; try again.\n"), stderr);
-		else {
+		} else {
 			strzero (cp);
 			break;
 		}
@@ -355,16 +367,18 @@ static void check_password (const struct passwd *pw, const struct spwd *sp)
 	 * If not expired and the "change only if expired" option (idea from
 	 * PAM) was specified, do nothing. --marekm
 	 */
-	if (kflg && exp_status == 0)
+	if (kflg && (0 == exp_status)) {
 		exit (E_SUCCESS);
+	}
 
 	/*
 	 * Root can change any password any time.
 	 */
-	if (amroot)
+	if (amroot) {
 		return;
+	}
 
-	time (&now);
+	(void) time (&now);
 
 	/*
 	 * Expired accounts cannot be changed ever. Passwords which are
@@ -372,12 +386,14 @@ static void check_password (const struct passwd *pw, const struct spwd *sp)
 	 * changed. Passwords which have been inactive too long cannot be
 	 * changed.
 	 */
-	if (sp->sp_pwdp[0] == '!' || exp_status > 1 ||
-	    (sp->sp_max >= 0 && sp->sp_min > sp->sp_max)) {
+	if (   (sp->sp_pwdp[0] == '!')
+	    || (exp_status > 1)
+	    || (   (sp->sp_max >= 0)
+	        && (sp->sp_min > sp->sp_max))) {
 		fprintf (stderr,
-			 _("The password for %s cannot be changed.\n"),
-			 sp->sp_namp);
-		SYSLOG ((LOG_WARN, "password locked for `%s'", sp->sp_namp));
+		         _("The password for %s cannot be changed.\n"),
+		         sp->sp_namp);
+		SYSLOG ((LOG_WARN, "password locked for '%s'", sp->sp_namp));
 		closelog ();
 		exit (E_NOPERM);
 	}
@@ -385,17 +401,18 @@ static void check_password (const struct passwd *pw, const struct spwd *sp)
 	/*
 	 * Passwords may only be changed after sp_min time is up.
 	 */
-	last = sp->sp_lstchg * SCALE;
-	ok = last + (sp->sp_min > 0 ? sp->sp_min * SCALE : 0);
+	if (sp->sp_lstchg > 0) {
+		last = sp->sp_lstchg * SCALE;
+		ok = last + (sp->sp_min > 0 ? sp->sp_min * SCALE : 0);
 
-	if (now < ok) {
-		fprintf (stderr,
-			 _
-			 ("The password for %s cannot be changed yet.\n"),
-			 pw->pw_name);
-		SYSLOG ((LOG_WARN, "now < minimum age for `%s'", pw->pw_name));
-		closelog ();
-		exit (E_NOPERM);
+		if (now < ok) {
+			fprintf (stderr,
+			         _("The password for %s cannot be changed yet.\n"),
+			         pw->pw_name);
+			SYSLOG ((LOG_WARN, "now < minimum age for '%s'", pw->pw_name));
+			closelog ();
+			exit (E_NOPERM);
+		}
 	}
 }
 
@@ -426,10 +443,12 @@ static char *date_to_str (time_t t)
 
 static const char *pw_status (const char *pass)
 {
-	if (*pass == '*' || *pass == '!')
+	if (*pass == '*' || *pass == '!') {
 		return "L";
-	if (*pass == '\0')
+	}
+	if (*pass == '\0') {
 		return "NP";
+	}
 	return "P";
 }
 
@@ -441,7 +460,7 @@ static void print_status (const struct passwd *pw)
 	struct spwd *sp;
 
 	sp = getspnam (pw->pw_name); /* local, no need for xgetspnam */
-	if (sp) {
+	if (NULL != sp) {
 		printf ("%s %s %s %ld %ld %ld %ld\n",
 			pw->pw_name,
 			pw_status (sp->sp_pwdp),
@@ -458,8 +477,22 @@ static void print_status (const struct passwd *pw)
 
 static void fail_exit (int status)
 {
-	pw_unlock ();
-	spw_unlock ();
+	if (pw_locked) {
+		if (pw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+			/* continue */
+		}
+	}
+
+	if (spw_locked) {
+		if (spw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
+			/* continue */
+		}
+	}
+
 	exit (status);
 }
 
@@ -472,18 +505,20 @@ static void oom (void)
 static char *update_crypt_pw (char *cp)
 {
 #ifndef USE_PAM
-	if (do_update_pwd)
+	if (do_update_pwd) {
 		cp = insert_crypt_passwd (cp, crypt_passwd);
+	}
 #endif
 
-	if (dflg)
+	if (dflg) {
 		*cp = '\0';
+	}
 
 	if (uflg && *cp == '!') {
 		if (cp[1] == '\0') {
 			fprintf (stderr,
-				 _("%s: unlocking the user would result in a passwordless account.\n"
-				   "You should set a password with usermod -p to unlock this user account.\n"),
+				 _("%s: unlocking the password would result in a passwordless account.\n"
+				   "You should set a password with usermod -p to unlock the password of this account.\n"),
 				 Prog);
 		} else {
 			cp++;
@@ -506,38 +541,51 @@ static void update_noshadow (void)
 	const struct passwd *pw;
 	struct passwd *npw;
 
-	if (!pw_lock ()) {
-		fputs (_("Cannot lock the password file; try again later.\n"),
-		       stderr);
-		SYSLOG ((LOG_WARN, "can't lock password file"));
+	if (pw_lock () == 0) {
+		fprintf (stderr,
+		         _("%s: cannot lock %s; try again later.\n"),
+		         Prog, pw_dbname ());
 		exit (E_PWDBUSY);
 	}
-	if (!pw_open (O_RDWR)) {
-		fputs (_("Cannot open the password file.\n"), stderr);
-		SYSLOG ((LOG_ERR, "can't open password file"));
+	pw_locked = true;
+	if (pw_open (O_RDWR) == 0) {
+		fprintf (stderr,
+		         _("%s: cannot open %s\n"),
+		         Prog, pw_dbname ());
+		SYSLOG ((LOG_WARN, "cannot open %s", pw_dbname ()));
 		fail_exit (E_MISSING);
 	}
 	pw = pw_locate (name);
-	if (!pw) {
-		fprintf (stderr, _("%s: %s not found in /etc/passwd\n"),
-			 Prog, name);
+	if (NULL == pw) {
+		fprintf (stderr,
+		         _("%s: user '%s' does not exist in %s\n"),
+		         Prog, name, pw_dbname ());
 		fail_exit (E_NOPERM);
 	}
 	npw = __pw_dup (pw);
-	if (!npw)
+	if (NULL == npw) {
 		oom ();
+	}
 	npw->pw_passwd = update_crypt_pw (npw->pw_passwd);
-	if (!pw_update (npw)) {
-		fputs (_("Error updating the password entry.\n"), stderr);
-		SYSLOG ((LOG_ERR, "error updating password entry"));
+	if (pw_update (npw) == 0) {
+		fprintf (stderr,
+		         _("%s: failed to prepare the new %s entry '%s'\n"),
+		         Prog, pw_dbname (), npw->pw_name);
 		fail_exit (E_FAILURE);
 	}
-	if (!pw_close ()) {
-		fputs (_("Cannot commit password file changes.\n"), stderr);
-		SYSLOG ((LOG_ERR, "can't rewrite password file"));
+	if (pw_close () == 0) {
+		fprintf (stderr,
+		         _("%s: failure while writing changes to %s\n"),
+		         Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", pw_dbname ()));
 		fail_exit (E_FAILURE);
 	}
-	pw_unlock ();
+	if (pw_unlock () == 0) {
+		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+		/* continue */
+	}
+	pw_locked = false;
 }
 
 static void update_shadow (void)
@@ -545,82 +593,87 @@ static void update_shadow (void)
 	const struct spwd *sp;
 	struct spwd *nsp;
 
-	if (!spw_lock ()) {
-		fputs (_("Cannot lock the password file; try again later.\n"),
-		       stderr);
-		SYSLOG ((LOG_WARN, "can't lock password file"));
+	if (spw_lock () == 0) {
+		fprintf (stderr,
+		         _("%s: cannot lock %s; try again later.\n"),
+		         Prog, spw_dbname ());
 		exit (E_PWDBUSY);
 	}
-	if (!spw_open (O_RDWR)) {
-		fputs (_("Cannot open the password file.\n"), stderr);
-		SYSLOG ((LOG_ERR, "can't open password file"));
+	spw_locked = true;
+	if (spw_open (O_RDWR) == 0) {
+		fprintf (stderr, _("%s: cannot open %s\n"), Prog, spw_dbname ());
+		SYSLOG ((LOG_WARN, "cannot open %s", spw_dbname ()));
 		fail_exit (E_FAILURE);
 	}
 	sp = spw_locate (name);
-	if (!sp) {
+	if (NULL == sp) {
 		/* Try to update the password in /etc/passwd instead. */
-		spw_close ();
+		(void) spw_close ();
 		update_noshadow ();
-		spw_unlock ();
+		if (spw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
+			/* continue */
+		}
+		spw_locked = false;
 		return;
 	}
 	nsp = __spw_dup (sp);
-	if (!nsp)
+	if (NULL == nsp) {
 		oom ();
-	nsp->sp_pwdp = update_crypt_pw (nsp->sp_pwdp);
-	if (xflg)
-		nsp->sp_max = (age_max * DAY) / SCALE;
-	if (nflg)
-		nsp->sp_min = (age_min * DAY) / SCALE;
-	if (wflg)
-		nsp->sp_warn = (warn * DAY) / SCALE;
-	if (iflg)
-		nsp->sp_inact = (inact * DAY) / SCALE;
-	if (do_update_age)
-		nsp->sp_lstchg = time ((time_t *) 0) / SCALE;
-	if (lflg) {
-		/* Set the account expiry field to 1.
-		 * Some PAM implementation consider zero as a non expired
-		 * account.
-		 */
-		nsp->sp_expire = 1;
 	}
-	if (uflg)
-		nsp->sp_expire = -1;
+	nsp->sp_pwdp = update_crypt_pw (nsp->sp_pwdp);
+	if (xflg) {
+		nsp->sp_max = (age_max * DAY) / SCALE;
+	}
+	if (nflg) {
+		nsp->sp_min = (age_min * DAY) / SCALE;
+	}
+	if (wflg) {
+		nsp->sp_warn = (warn * DAY) / SCALE;
+	}
+	if (iflg) {
+		nsp->sp_inact = (inact * DAY) / SCALE;
+	}
+#ifndef USE_PAM
+	if (do_update_age) {
+		nsp->sp_lstchg = (long) time ((time_t *) 0) / SCALE;
+		if (0 == nsp->sp_lstchg) {
+			/* Better disable aging than requiring a password
+			 * change */
+			nsp->sp_lstchg = -1;
+		}
+	}
+#endif
 
 	/*
 	 * Force change on next login, like SunOS 4.x passwd -e or Solaris
 	 * 2.x passwd -f. Solaris 2.x seems to do the same thing (set
 	 * sp_lstchg to 0).
 	 */
-	if (eflg)
+	if (eflg) {
 		nsp->sp_lstchg = 0;
+	}
 
-	if (!spw_update (nsp)) {
-		fputs (_("Error updating the password entry.\n"), stderr);
-		SYSLOG ((LOG_ERR, "error updating password entry"));
+	if (spw_update (nsp) == 0) {
+		fprintf (stderr,
+		         _("%s: failed to prepare the new %s entry '%s'\n"),
+		         Prog, spw_dbname (), nsp->sp_namp);
 		fail_exit (E_FAILURE);
 	}
-	if (!spw_close ()) {
-		fputs (_("Cannot commit password file changes.\n"), stderr);
-		SYSLOG ((LOG_ERR, "can't rewrite password file"));
+	if (spw_close () == 0) {
+		fprintf (stderr,
+		         _("%s: failure while writing changes to %s\n"),
+		         Prog, spw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", spw_dbname ()));
 		fail_exit (E_FAILURE);
 	}
-	spw_unlock ();
-}
-
-static long getnumber (const char *numstr)
-{
-	long val;
-	char *errptr;
-
-	val = strtol (numstr, &errptr, 10);
-	if (*errptr || errno == ERANGE) {
-		fprintf (stderr, _("%s: invalid numeric argument '%s'\n"), Prog,
-			 numstr);
-		exit (E_BAD_ARG);
+	if (spw_unlock () == 0) {
+		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
+		SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
+		/* continue */
 	}
-	return val;
+	spw_locked = false;
 }
 
 #ifdef WITH_SELINUX
@@ -630,19 +683,21 @@ static int check_selinux_access (const char *changed_user,
 {
 	int status = -1;
 	security_context_t user_context;
+	context_t c;
 	const char *user;
 
 	/* if in permissive mode then allow the operation */
-	if (security_getenforce() == 0)
+	if (security_getenforce() == 0) {
 		return 0;
+	}
 
 	/* get the context of the process which executed passwd */
-	if (getprevcon(&user_context))
+	if (getprevcon(&user_context) != 0) {
 		return -1;
+	}
 
 	/* get the "user" portion of the context (the part before the first
 	   colon) */
-	context_t c;
 	c = context_new(user_context);
 	user = context_user_get(c);
 
@@ -684,12 +739,12 @@ static int check_selinux_access (const char *changed_user,
  *	-g	execute gpasswd command to interpret flags
  *	-i #	set sp_inact to # days (*)
  *	-k	change password only if expired
- *	-l	lock the named account (*)
+ *	-l	lock the password of the named account (*)
  *	-n #	set sp_min to # days (*)
  *	-r #	change password in # repository
  *	-s	execute chsh command to interpret flags
  *	-S	show password status of named account
- *	-u	unlock the named account (*)
+ *	-u	unlock the password of the named account (*)
  *	-w #	set sp_warn to # days (*)
  *	-x #	set sp_max to # days (*)
  *
@@ -709,9 +764,9 @@ int main (int argc, char **argv)
 	const struct spwd *sp;	/* Shadow file entry for user   */
 #endif
 
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
+	(void) setlocale (LC_ALL, "");
+	(void) bindtextdomain (PACKAGE, LOCALEDIR);
+	(void) textdomain (PACKAGE);
 
 	/*
 	 * The program behaves differently when executed by root than when
@@ -758,66 +813,87 @@ int main (int argc, char **argv)
 				     long_options, &option_index)) != -1) {
 			switch (c) {
 			case 'a':
-				aflg++;
+				aflg = true;
 				break;
 			case 'd':
-				dflg++;
-				anyflag = 1;
+				dflg = true;
+				anyflag = true;
 				break;
 			case 'e':
-				eflg++;
-				anyflag = 1;
+				eflg = true;
+				anyflag = true;
 				break;
 			case 'i':
-				inact = getnumber (optarg);
-				if (inact >= -1)
-					iflg++;
-				anyflag = 1;
+				if (   (getlong (optarg, &inact) == 0)
+				    || (inact < -1)) {
+					fprintf (stderr,
+					         _("%s: invalid numeric argument '%s'\n"),
+					         Prog, optarg);
+					usage (E_BAD_ARG);
+				}
+				iflg = true;
+				anyflag = true;
 				break;
 			case 'k':
 				/* change only if expired, like Linux-PAM passwd -k. */
-				kflg++;	/* ok for users */
+				kflg = true;	/* ok for users */
 				break;
 			case 'l':
-				lflg++;
-				anyflag = 1;
+				lflg = true;
+				anyflag = true;
 				break;
 			case 'n':
-				age_min = getnumber (optarg);
-				nflg++;
-				anyflag = 1;
+				if (   (getlong (optarg, &age_min) == 0)
+				    || (age_min < -1)) {
+					fprintf (stderr,
+					         _("%s: invalid numeric argument '%s'\n"),
+					         Prog, optarg);
+					usage (E_BAD_ARG);
+				}
+				nflg = true;
+				anyflag = true;
 				break;
 			case 'q':
-				qflg++;	/* ok for users */
+				qflg = true;	/* ok for users */
 				break;
 			case 'r':
 				/* -r repository (files|nis|nisplus) */
 				/* only "files" supported for now */
 				if (strcmp (optarg, "files") != 0) {
 					fprintf (stderr,
-						 _
-						 ("%s: repository %s not supported\n"),
+					         _("%s: repository %s not supported\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
 				break;
 			case 'S':
-				Sflg++;	/* ok for users */
+				Sflg = true;	/* ok for users */
 				break;
 			case 'u':
-				uflg++;
-				anyflag = 1;
+				uflg = true;
+				anyflag = true;
 				break;
 			case 'w':
-				warn = getnumber (optarg);
-				if (warn >= -1)
-					wflg++;
-				anyflag = 1;
+				if (   (getlong (optarg, &warn) == 0)
+				    || (warn < -1)) {
+					fprintf (stderr,
+					         _("%s: invalid numeric argument '%s'\n"),
+					         Prog, optarg);
+					usage (E_BAD_ARG);
+				}
+				wflg = true;
+				anyflag = true;
 				break;
 			case 'x':
-				age_max = getnumber (optarg);
-				xflg++;
-				anyflag = 1;
+				if (   (getlong (optarg, &age_max) == 0)
+				    || (age_max < -1)) {
+					fprintf (stderr,
+					         _("%s: invalid numeric argument '%s'\n"),
+					         Prog, optarg);
+					usage (E_BAD_ARG);
+				}
+				xflg = true;
+				anyflag = true;
 				break;
 			default:
 				usage (E_BAD_ARG);
@@ -831,30 +907,35 @@ int main (int argc, char **argv)
 	 * environment.
 	 */
 	pw = get_my_pwent ();
-	if (!pw) {
+	if (NULL == pw) {
 		fprintf (stderr,
-			 _("%s: Cannot determine your user name.\n"), Prog);
+		         _("%s: Cannot determine your user name.\n"), Prog);
+		SYSLOG ((LOG_WARN, "Cannot determine the user name of the caller (UID %lu)",
+		         (unsigned long) getuid ()));
 		exit (E_NOPERM);
 	}
 	myname = xstrdup (pw->pw_name);
-	if (optind < argc)
+	if (optind < argc) {
 		name = argv[optind];
-	else
+	} else {
 		name = myname;
+	}
 
 	/*
 	 * Make sure that at most one username was specified.
 	 */
-	if (argc > optind+1)
+	if (argc > (optind+1)) {
 		usage (E_USAGE);
+	}
 
 	/*
 	 * The -a flag requires -S, no other flags, no username, and
 	 * you must be root.  --marekm
 	 */
 	if (aflg) {
-		if (anyflag || !Sflg || (optind < argc))
+		if (anyflag || !Sflg || (optind < argc)) {
 			usage (E_USAGE);
+		}
 		if (!amroot) {
 			fprintf (stderr, _("%s: Permission denied.\n"), Prog);
 			exit (E_NOPERM);
@@ -886,11 +967,14 @@ int main (int argc, char **argv)
 	 * -S now ok for normal users (check status of my own account), and
 	 * doesn't require username.  --marekm
 	 */
-	if (anyflag && optind >= argc)
+	if (anyflag && optind >= argc) {
 		usage (E_USAGE);
+	}
 
-	if (anyflag + Sflg + kflg > 1)
+	if (   (Sflg && kflg)
+	    || (anyflag && (Sflg || kflg))) {
 		usage (E_USAGE);
+	}
 
 	if (anyflag && !amroot) {
 		fprintf (stderr, _("%s: Permission denied.\n"), Prog);
@@ -898,8 +982,8 @@ int main (int argc, char **argv)
 	}
 
 	pw = xgetpwnam (name);
-	if (!pw) {
-		fprintf (stderr, _("%s: unknown user %s\n"), Prog, name);
+	if (NULL == pw) {
+		fprintf (stderr, _("%s: user '%s' does not exist\n"), Prog, name);
 		exit (E_NOPERM);
 	}
 #ifdef WITH_SELINUX
@@ -926,11 +1010,10 @@ int main (int argc, char **argv)
 	 * If the UID of the user does not match the current real UID,
 	 * check if I'm root.
 	 */
-	if (!amroot && pw->pw_uid != getuid ()) {
+	if (!amroot && (pw->pw_uid != getuid ())) {
 		fprintf (stderr,
-			 _
-			 ("%s: You may not view or modify password information for %s.\n"),
-			 Prog, name);
+		         _("%s: You may not view or modify password information for %s.\n"),
+		         Prog, name);
 		SYSLOG ((LOG_WARN,
 			 "%s: can't view or modify password information for %s",
 			 Prog, name));
@@ -947,8 +1030,9 @@ int main (int argc, char **argv)
 	 * The user name is valid, so let's get the shadow file entry.
 	 */
 	sp = getspnam (name); /* !USE_PAM, no need for xgetspnam */
-	if (!sp)
+	if (NULL == sp) {
 		sp = pwd_to_spwd (pw);
+	}
 
 	cp = sp->sp_pwdp;
 
@@ -967,8 +1051,9 @@ int main (int argc, char **argv)
 		/*
 		 * Let the user know whose password is being changed.
 		 */
-		if (!qflg)
+		if (!qflg) {
 			printf (_("Changing password for %s\n"), name);
+		}
 
 		if (new_password (pw)) {
 			fprintf (stderr,
@@ -977,8 +1062,8 @@ int main (int argc, char **argv)
 			closelog ();
 			exit (E_NOPERM);
 		}
-		do_update_pwd = 1;
-		do_update_age = 1;
+		do_update_pwd = true;
+		do_update_age = true;
 	}
 #endif				/* !USE_PAM */
 	/*
@@ -998,28 +1083,31 @@ int main (int argc, char **argv)
 		exit (E_SUCCESS);
 	}
 #endif				/* USE_PAM */
-	if (setuid (0)) {
+	if (setuid (0) != 0) {
 		fputs (_("Cannot change ID to root.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
 		closelog ();
 		exit (E_NOPERM);
 	}
-	if (spw_file_present ())
+	if (spw_file_present ()) {
 		update_shadow ();
-	else
+	} else {
 		update_noshadow ();
+	}
 
 	nscd_flush_cache ("passwd");
 	nscd_flush_cache ("group");
 
-	SYSLOG ((LOG_INFO, "password for `%s' changed by `%s'", name, myname));
+	SYSLOG ((LOG_INFO, "password for '%s' changed by '%s'", name, myname));
 	closelog ();
 	if (!qflg) {
-		if (!eflg)
+		if (!eflg) {
 			puts (_("Password changed."));
-		else
+		} else {
 			puts (_("Password set to expire."));
+		}
 	}
 	exit (E_SUCCESS);
 	/* NOT REACHED */
 }
+
