@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 1996 - 2000, Marek Michałkiewicz
  * Copyright (c) 2002 - 2006, Tomasz Kłoczko
+ * Copyright (c) 2009       , Nicolas François
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,7 +55,7 @@
 
 #include <config.h>
 
-#ident "$Id: pwconv.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+#ident "$Id: pwconv.c 2589 2009-04-05 21:23:42Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -83,18 +84,32 @@
 /*
  * Global variables
  */
-static int
- shadow_locked = 0, passwd_locked = 0;
+char *Prog;
+
+static bool spw_locked = false;
+static bool pw_locked = false;
 
 /* local function prototypes */
-static void fail_exit (int);
+static void fail_exit (int status);
 
 static void fail_exit (int status)
 {
-	if (shadow_locked)
-		spw_unlock ();
-	if (passwd_locked)
-		pw_unlock ();
+	if (pw_locked) {
+		if (pw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+			/* continue */
+		}
+	}
+
+	if (spw_locked) {
+		if (spw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
+			/* continue */
+		}
+	}
+
 	exit (status);
 }
 
@@ -104,29 +119,41 @@ int main (int argc, char **argv)
 	struct passwd pwent;
 	const struct spwd *sp;
 	struct spwd spent;
-	char *Prog = argv[0];
 
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
+	if (1 != argc) {
+		(void) fputs (_("Usage: pwconv\n"), stderr);
+	}
+	Prog = Basename (argv[0]);
 
-	if (!pw_lock ()) {
-		fprintf (stderr, _("%s: can't lock passwd file\n"), Prog);
+	(void) setlocale (LC_ALL, "");
+	(void) bindtextdomain (PACKAGE, LOCALEDIR);
+	(void) textdomain (PACKAGE);
+
+	OPENLOG ("pwconv");
+
+	if (pw_lock () == 0) {
+		fprintf (stderr,
+		         _("%s: cannot lock %s; try again later.\n"),
+		         Prog, pw_dbname ());
 		fail_exit (E_PWDBUSY);
 	}
-	passwd_locked++;
-	if (!pw_open (O_RDWR)) {
-		fprintf (stderr, _("%s: can't open passwd file\n"), Prog);
+	pw_locked = true;
+	if (pw_open (O_RDWR) == 0) {
+		fprintf (stderr,
+		         _("%s: cannot open %s\n"), Prog, pw_dbname ());
 		fail_exit (E_MISSING);
 	}
 
-	if (!spw_lock ()) {
-		fprintf (stderr, _("%s: can't lock shadow file\n"), Prog);
+	if (spw_lock () == 0) {
+		fprintf (stderr,
+		         _("%s: cannot lock %s; try again later.\n"),
+		         Prog, spw_dbname ());
 		fail_exit (E_PWDBUSY);
 	}
-	shadow_locked++;
-	if (!spw_open (O_CREAT | O_RDWR)) {
-		fprintf (stderr, _("%s: can't open shadow file\n"), Prog);
+	spw_locked = true;
+	if (spw_open (O_CREAT | O_RDWR) == 0) {
+		fprintf (stderr,
+		         _("%s: cannot open %s\n"), Prog, spw_dbname ());
 		fail_exit (E_FAILURE);
 	}
 
@@ -134,18 +161,18 @@ int main (int argc, char **argv)
 	 * Remove /etc/shadow entries for users not in /etc/passwd.
 	 */
 	spw_rewind ();
-	while ((sp = spw_next ())) {
-		if (pw_locate (sp->sp_namp))
+	while ((sp = spw_next ()) != NULL) {
+		if (pw_locate (sp->sp_namp) != NULL) {
 			continue;
+		}
 
-		if (!spw_remove (sp->sp_namp)) {
+		if (spw_remove (sp->sp_namp) == 0) {
 			/*
 			 * This shouldn't happen (the entry exists) but...
 			 */
 			fprintf (stderr,
-				 _
-				 ("%s: can't remove shadow entry for %s\n"),
-				 Prog, sp->sp_namp);
+			         _("%s: cannot remove entry '%s' from %s\n"),
+			         Prog, sp->sp_namp, spw_dbname ());
 			fail_exit (E_FAILURE);
 		}
 	}
@@ -155,60 +182,93 @@ int main (int argc, char **argv)
 	 * missing shadow entries.
 	 */
 	pw_rewind ();
-	while ((pw = pw_next ())) {
+	while ((pw = pw_next ()) != NULL) {
 		sp = spw_locate (pw->pw_name);
-		if (sp) {
+		if (NULL != sp) {
 			/* do we need to update this entry? */
-			if (strcmp (pw->pw_passwd, SHADOW_PASSWD_STRING) == 0)
+			if (strcmp (pw->pw_passwd, SHADOW_PASSWD_STRING) == 0) {
 				continue;
+			}
 			/* update existing shadow entry */
 			spent = *sp;
 		} else {
 			/* add new shadow entry */
 			memset (&spent, 0, sizeof spent);
-			spent.sp_namp = pw->pw_name;
-			spent.sp_min = getdef_num ("PASS_MIN_DAYS", -1);
-			spent.sp_max = getdef_num ("PASS_MAX_DAYS", -1);
-			spent.sp_warn = getdef_num ("PASS_WARN_AGE", -1);
-			spent.sp_inact = -1;
+			spent.sp_namp   = pw->pw_name;
+			spent.sp_min    = getdef_num ("PASS_MIN_DAYS", -1);
+			spent.sp_max    = getdef_num ("PASS_MAX_DAYS", -1);
+			spent.sp_warn   = getdef_num ("PASS_WARN_AGE", -1);
+			spent.sp_inact  = -1;
 			spent.sp_expire = -1;
-			spent.sp_flag = -1;
+			spent.sp_flag   = SHADOW_SP_FLAG_UNSET;
 		}
 		spent.sp_pwdp = pw->pw_passwd;
-		spent.sp_lstchg = time ((time_t *) 0) / (24L * 3600L);
-		if (!spw_update (&spent)) {
+		spent.sp_lstchg = (long) time ((time_t *) 0) / SCALE;
+		if (0 == spent.sp_lstchg) {
+			/* Better disable aging than requiring a password
+			 * change */
+			spent.sp_lstchg = -1;
+		}
+		if (spw_update (&spent) == 0) {
 			fprintf (stderr,
-				 _
-				 ("%s: can't update shadow entry for %s\n"),
-				 Prog, spent.sp_namp);
+			         _("%s: failed to prepare the new %s entry '%s'\n"),
+			         Prog, spw_dbname (), spent.sp_namp);
 			fail_exit (E_FAILURE);
 		}
 
 		/* remove password from /etc/passwd */
 		pwent = *pw;
 		pwent.pw_passwd = SHADOW_PASSWD_STRING;	/* XXX warning: const */
-		if (!pw_update (&pwent)) {
+		if (pw_update (&pwent) == 0) {
 			fprintf (stderr,
-				 _
-				 ("%s: can't update passwd entry for %s\n"),
-				 Prog, pwent.pw_name);
+			         _("%s: failed to prepare the new %s entry '%s'\n"),
+			         Prog, pw_dbname (), pwent.pw_name);
 			fail_exit (E_FAILURE);
 		}
 	}
 
-	if (!spw_close ()) {
-		fprintf (stderr, _("%s: can't update shadow file\n"), Prog);
+	if (spw_close () == 0) {
+		fprintf (stderr,
+		         _("%s: failure while writing changes to %s\n"),
+		         Prog, spw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", spw_dbname ()));
 		fail_exit (E_FAILURE);
 	}
-	if (!pw_close ()) {
-		fprintf (stderr, _("%s: can't update passwd file\n"), Prog);
+	if (pw_close () == 0) {
+		fprintf (stderr,
+		         _("%s: failure while writing changes to %s\n"),
+		         Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", pw_dbname ()));
 		fail_exit (E_FAILURE);
 	}
-	chmod (PASSWD_FILE "-", 0600);	/* /etc/passwd- (backup file) */
-	spw_unlock ();
-	pw_unlock ();
+
+	/* /etc/passwd- (backup file) */
+	if (chmod (PASSWD_FILE "-", 0600) != 0) {
+		fprintf (stderr,
+		         _("%s: failed to change the mode of %s to 0600\n"),
+		         Prog, PASSWD_FILE "-");
+		SYSLOG ((LOG_ERR, "failed to change the mode of %s to 0600", PASSWD_FILE "-"));
+		/* continue */
+	}
+
+	if (pw_locked) {
+		if (pw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+			/* continue */
+		}
+	}
+
+	if (spw_locked) {
+		if (spw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
+			/* continue */
+		}
+	}
 
 	nscd_flush_cache ("passwd");
 
 	exit (E_SUCCESS);
 }
+
