@@ -32,7 +32,7 @@
 
 #include <config.h>
 
-#ident "$Id: pwunconv.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+#ident "$Id: pwunconv.c 2348 2008-09-06 12:51:53Z nekral-guest $"
 
 #include <fcntl.h>
 #include <pwd.h>
@@ -47,17 +47,30 @@
 /*
  * Global variables
  */
-static int shadow_locked = 0, passwd_locked = 0;
+char *Prog;
+
+static bool spw_locked = false;
+static bool pw_locked = false;
 
 /* local function prototypes */
-static void fail_exit (int);
+static void fail_exit (int status);
 
 static void fail_exit (int status)
 {
-	if (shadow_locked)
-		spw_unlock ();
-	if (passwd_locked)
-		pw_unlock ();
+	if (spw_locked) {
+		if (spw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
+			/* continue */
+		}
+	}
+	if (pw_locked) {
+		if (pw_unlock () == 0) {
+			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+			/* continue */
+		}
+	}
 	exit (status);
 }
 
@@ -68,48 +81,65 @@ int main (int argc, char **argv)
 	struct passwd pwent;
 	const struct spwd *spwd;
 
-	char *Prog = argv[0];
+	if (1 != argc) {
+		(void) fputs (_("Usage: pwunconv\n"), stderr);
+	}
+	Prog = Basename (argv[0]);
 
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
+	(void) setlocale (LC_ALL, "");
+	(void) bindtextdomain (PACKAGE, LOCALEDIR);
+	(void) textdomain (PACKAGE);
 
-	if (!spw_file_present ())
+	OPENLOG ("pwunconv");
+
+	if (!spw_file_present ()) {
 		/* shadow not installed, do nothing */
 		exit (0);
+	}
 
-	if (!pw_lock ()) {
-		fprintf (stderr, _("%s: can't lock passwd file\n"), Prog);
+	if (pw_lock () == 0) {
+		fprintf (stderr,
+		         _("%s: cannot lock %s; try again later.\n"),
+		         Prog, pw_dbname ());
 		fail_exit (5);
 	}
-	passwd_locked++;
-	if (!pw_open (O_RDWR)) {
-		fprintf (stderr, _("%s: can't open passwd file\n"), Prog);
+	pw_locked = true;
+	if (pw_open (O_RDWR) == 0) {
+		fprintf (stderr,
+		         _("%s: cannot open %s\n"),
+		         Prog, pw_dbname ());
 		fail_exit (1);
 	}
 
-	if (!spw_lock ()) {
-		fprintf (stderr, _("%s: can't lock shadow file\n"), Prog);
+	if (spw_lock () == 0) {
+		fprintf (stderr,
+		         _("%s: cannot lock %s; try again later.\n"),
+		         Prog, spw_dbname ());
 		fail_exit (5);
 	}
-	shadow_locked++;
-	if (!spw_open (O_RDWR)) {
-		fprintf (stderr, _("%s: can't open shadow file\n"), Prog);
+	spw_locked = true;
+	if (spw_open (O_RDWR) == 0) {
+		fprintf (stderr,
+		         _("%s: cannot open %s\n"),
+		         Prog, spw_dbname ());
 		fail_exit (1);
 	}
 
 	pw_rewind ();
-	while ((pw = pw_next ())) {
-		if (!(spwd = spw_locate (pw->pw_name)))
+	while ((pw = pw_next ()) != NULL) {
+		spwd = spw_locate (pw->pw_name);
+		if (NULL == spwd) {
 			continue;
+		}
 
 		pwent = *pw;
 
 		/*
 		 * Update password if non-shadow is "x".
 		 */
-		if (strcmp (pw->pw_passwd, SHADOW_PASSWD_STRING) == 0)
+		if (strcmp (pw->pw_passwd, SHADOW_PASSWD_STRING) == 0) {
 			pwent.pw_passwd = spwd->sp_pwdp;
+		}
 
 		/*
 		 * Password aging works differently in the two different
@@ -121,35 +151,50 @@ int main (int argc, char **argv)
 		 * put into the new file. Otherwise, the days are converted
 		 * to weeks and so on.
 		 */
-		if (!pw_update (&pwent)) {
+		if (pw_update (&pwent) == 0) {
 			fprintf (stderr,
-				 _("%s: can't update entry for user %s\n"),
-				 Prog, pwent.pw_name);
+			         _("%s: failed to prepare the new %s entry '%s'\n"),
+			         Prog, pw_dbname (), pwent.pw_name);
 			fail_exit (3);
 		}
 	}
 
-	if (!spw_close ()) {
+	if (spw_close () == 0) {
 		fprintf (stderr,
-			 _("%s: can't update shadow password file\n"), Prog);
+		         _("%s: failure while writing changes to %s\n"),
+		         Prog, spw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", spw_dbname ()));
 		fail_exit (3);
 	}
 
-	if (!pw_close ()) {
-		fprintf (stderr, _("%s: can't update password file\n"), Prog);
+	if (pw_close () == 0) {
+		fprintf (stderr,
+		         _("%s: failure while writing changes to %s\n"),
+		         Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failure while writing changes to %s", pw_dbname ()));
 		fail_exit (3);
 	}
 
 	if (unlink (SHADOW) != 0) {
 		fprintf (stderr,
-			 _("%s: can't delete shadow password file\n"), Prog);
+			 _("%s: cannot delete %s\n"), Prog, SHADOW);
+		SYSLOG ((LOG_ERR, "cannot delete %s", SHADOW));
 		fail_exit (3);
 	}
 
-	spw_unlock ();
-	pw_unlock ();
+	if (spw_unlock () == 0) {
+		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
+		SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
+		/* continue */
+	}
+	if (pw_unlock () == 0) {
+		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
+		SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
+		/* continue */
+	}
 
 	nscd_flush_cache ("passwd");
 
 	return 0;
 }
+
